@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -35,27 +35,17 @@ bool ispow2(int n)
 
 int nextpow2(int n)
 {
-    int ret=1;
-    while (ret < n) {
-        ret *= 2;
-    }
-    return ret;
-/* TODO: Fix this fast version :-).
-    int RetVal = 1;
-    __asm__ __volatile__(
-        "xorl %%ecx, %%ecx\n\t"
-        "bsrl %1, %%ecx\n\t"
-        "incl %%ecx\n\t"
-        "shlb %%cl, %0\n\t"
-        : "=m" (RetVal)
-        : "m" (n)
-        : "cc", "ecx"
-        );
-    return RetVal;
-*/    
+    n--;
+    n |= n >> 1;  // handle  2 bit numbers
+    n |= n >> 2;  // handle  4 bit numbers
+    n |= n >> 4;  // handle  8 bit numbers
+    n |= n >> 8;  // handle 16 bit numbers
+    n |= n >> 16; // handle 32 bit numbers
+    n++;
+    return n;
 }
 
-int safeCeil(double d) 
+int safeCeil(float d) 
 {
     if (fabs(d-int(d)) < EPSILON) {
         return int(d);
@@ -80,7 +70,7 @@ float invSqrt(float x)
     return 1/sqrt(x);
 }
 
-bool almostEqual(double d1, double d2, double epsilon)
+bool almostEqual(float d1, float d2, float epsilon)
 {
     return (fabs(d1-d2)<epsilon);
 }
