@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -42,6 +42,10 @@ size_t getMemoryUsage();
 // Under other operating systems, returns the input string.
 AVG_API std::string convertUTF8ToFilename(const std::string & sName);
 
+#ifdef __APPLE__
+int getOSXMajorVersion();
+#endif
+
 }
 
 #endif 
