@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,12 +30,11 @@ using namespace std;
 namespace avg {
 
 ProfilingZone::ProfilingZone(const ProfilingZoneID& zoneID)
-    : m_ZoneID(zoneID),
-      m_TimeSum(0),
+    : m_TimeSum(0),
       m_AvgTime(0),
       m_NumFrames(0),
       m_Indent(0),
-      m_bIsRegistered(false)
+      m_ZoneID(zoneID)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
 }
@@ -45,6 +44,13 @@ ProfilingZone::~ProfilingZone()
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
+void ProfilingZone::restart()
+{
+    m_NumFrames = 0;
+    m_AvgTime = 0;
+    m_TimeSum = 0;
+}
+
 void ProfilingZone::reset()
 {
     m_NumFrames++;
