@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -44,7 +44,7 @@ void FilterNormalize::applyInPlace(BitmapPtr pBmp)
         min -= 2;
         max += 2;
     }
-    double factor = 255./(max-min);
+    float factor = 255.f/(max-min);
     if (factor > 10) {
         factor = 10;
     }
