@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,13 +33,13 @@ namespace avg {
 class AVG_API FilterColorize : public Filter
 {
 public:
-  FilterColorize(double hue, double saturation);
+  FilterColorize(float hue, float saturation);
   virtual ~FilterColorize();
   virtual void applyInPlace(BitmapPtr pBmp) ;
 
 private:
-  double m_Hue;
-  double m_Saturation;
+  float m_Hue;
+  float m_Saturation;
 };
 
 }  // namespace
