@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,10 +24,15 @@
 
 #include "../base/Exception.h"
 
+#include <iostream>
+
+using namespace std;
+
 namespace avg {
     
-FilterFlipRGB::FilterFlipRGB()
-  : Filter()
+FilterFlipRGB::FilterFlipRGB(bool bChangePF)
+    : Filter(),
+      m_bChangePF(bChangePF)
 {
 }
 
@@ -38,29 +43,32 @@ FilterFlipRGB::~FilterFlipRGB()
 
 void FilterFlipRGB::applyInPlace(BitmapPtr pBmp) 
 {
+    AVG_ASSERT(pBmp->getBytesPerPixel() >= 3);
     PixelFormat PF = pBmp->getPixelFormat();
-    switch(PF) {
-        case B8G8R8A8:
-            pBmp->setPixelFormat(R8G8B8A8);
-            break;
-        case B8G8R8X8:
-            pBmp->setPixelFormat(R8G8B8X8);
-            break;
-        case R8G8B8A8:
-            pBmp->setPixelFormat(B8G8R8A8);
-            break;
-        case R8G8B8X8:
-            pBmp->setPixelFormat(B8G8R8X8);
-            break;
-        case R8G8B8:
-            pBmp->setPixelFormat(B8G8R8);
-            break;
-        case B8G8R8:
-            pBmp->setPixelFormat(R8G8B8);
-            break;
-        default:
-            // Only 24 and 32 bpp supported.
-            AVG_ASSERT(false);
+    if (m_bChangePF) {
+        switch(PF) {
+            case B8G8R8A8:
+                pBmp->setPixelFormat(R8G8B8A8);
+                break;
+            case B8G8R8X8:
+                pBmp->setPixelFormat(R8G8B8X8);
+                break;
+            case R8G8B8A8:
+                pBmp->setPixelFormat(B8G8R8A8);
+                break;
+            case R8G8B8X8:
+                pBmp->setPixelFormat(B8G8R8X8);
+                break;
+            case R8G8B8:
+                pBmp->setPixelFormat(B8G8R8);
+                break;
+            case B8G8R8:
+                pBmp->setPixelFormat(R8G8B8);
+                break;
+            default:
+                // Only 24 and 32 bpp supported.
+                AVG_ASSERT(false);
+        }
     }
     IntPoint size = pBmp->getSize();
     for (int y = 0; y < size.y; y++) {
