@@ -1,5 +1,5 @@
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,21 +23,27 @@
 
 #include "../api.h"
 
+#include <string>
+
 namespace avg {
 
 struct AVG_API GLConfig {
+    enum ShaderUsage {FULL, MINIMAL, AUTO};
+
     GLConfig();
-    GLConfig(bool bUsePOTTextures, bool bUseShaders, bool bUsePixelBuffers,
-            int multiSampleSamples);
+    GLConfig(bool bGLES, bool bUsePOTTextures, bool bUsePixelBuffers,
+            int multiSampleSamples, ShaderUsage shaderUsage, bool bUseDebugContext);
 
     void log();
 
+    static std::string shaderUsageToString(ShaderUsage su);
+
+    bool m_bGLES;
     bool m_bUsePOTTextures;
-    bool m_bUseShaders;
     bool m_bUsePixelBuffers;
     int m_MultiSampleSamples;
-
-//    OGLMemoryMode m_MemoryMode;
+    ShaderUsage m_ShaderUsage;
+    bool m_bUseDebugContext;
 };
 
 }
