@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 
 #include "../api.h"
 #include "GPUFilter.h"
+#include "GLShaderParam.h"
 #include "GLTexture.h"
 
 namespace avg {
@@ -32,21 +33,32 @@ class AVG_API GPUBlurFilter: public GPUFilter
 {
 public:
     GPUBlurFilter(const IntPoint& size, PixelFormat pfSrc, PixelFormat pfDest, 
-            double stdDev, bool bClipBorders, bool bStandalone=true);
+            float stdDev, bool bClipBorders, bool bStandalone=true, 
+            bool bUseFloatKernel=false);
     virtual ~GPUBlurFilter();
     
-    void setStdDev(double stdDev);
+    void setStdDev(float stdDev);
     virtual void applyOnGPU(GLTexturePtr pSrcTex);
 
 private:
-    void initShaders();
-    void setDimensions(IntPoint size, double stdDev, bool bClipBorders);
+    void setDimensions(IntPoint size, float stdDev, bool bClipBorders);
 
-    double m_StdDev;
+    float m_StdDev;
     bool m_bClipBorders;
+    bool m_bUseFloatKernel;
 
     GLTexturePtr m_pGaussCurveTex;
     ImagingProjectionPtr m_pProjection2;
+
+    FloatGLShaderParamPtr m_pHorizWidthParam;
+    IntGLShaderParamPtr m_pHorizRadiusParam;
+    IntGLShaderParamPtr m_pHorizTextureParam;
+    IntGLShaderParamPtr m_pHorizKernelTexParam;
+
+    FloatGLShaderParamPtr m_pVertWidthParam;
+    IntGLShaderParamPtr m_pVertRadiusParam;
+    IntGLShaderParamPtr m_pVertTextureParam;
+    IntGLShaderParamPtr m_pVertKernelTexParam;
 };
 
 typedef boost::shared_ptr<GPUBlurFilter> GPUBlurFilterPtr;
