@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 
 #include "../api.h"
 #include "GPUFilter.h"
+#include "GLShaderParam.h"
 #include "Bitmap.h"
 
 namespace avg {
@@ -31,24 +32,39 @@ namespace avg {
 class AVG_API GPUChromaKeyFilter: public GPUFilter
 {
 public:
-    GPUChromaKeyFilter(const IntPoint& size, PixelFormat pf, bool bStandalone=true);
+    GPUChromaKeyFilter(const IntPoint& size, bool bStandalone=true);
     virtual ~GPUChromaKeyFilter();
 
-    void setParams(const Pixel32& color, double hTolerance, double sTolerance, 
-            double lTolerance, double softness, int erosion, double spillThreshold);
+    void setParams(const Pixel32& color, float hTolerance, float sTolerance, 
+            float lTolerance, float softness, int erosion, float spillThreshold);
 
     virtual void applyOnGPU(GLTexturePtr pSrcTex);
 
 private:
-    void initShader();
-
     Pixel32 m_Color;
-    double m_HTolerance;
-    double m_STolerance;
-    double m_LTolerance;
-    double m_Softness;
+    float m_HTolerance;
+    float m_STolerance;
+    float m_LTolerance;
+    float m_Softness;
     int m_Erosion;
-    double m_SpillThreshold;
+    float m_SpillThreshold;
+    ImagingProjectionPtr m_pProjection2;
+    
+    IntGLShaderParamPtr m_pTextureParam;
+    FloatGLShaderParamPtr m_pHKeyParam;
+    FloatGLShaderParamPtr m_pHToleranceParam;
+    FloatGLShaderParamPtr m_pHSoftToleranceParam;
+    FloatGLShaderParamPtr m_pSKeyParam;
+    FloatGLShaderParamPtr m_pSToleranceParam;
+    FloatGLShaderParamPtr m_pSSoftToleranceParam;
+    FloatGLShaderParamPtr m_pLKeyParam;
+    FloatGLShaderParamPtr m_pLToleranceParam;
+    FloatGLShaderParamPtr m_pLSoftToleranceParam;
+    FloatGLShaderParamPtr m_pSpillThresholdParam;
+    IntGLShaderParamPtr m_pIsLastParam;
+
+    IntGLShaderParamPtr m_pErosionTextureParam;
+    IntGLShaderParamPtr m_pErosionIsLastParam;
 };
 
 typedef boost::shared_ptr<GPUChromaKeyFilter> GPUChromaKeyFilterPtr;
