@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 
 #include "../api.h"
 #include "GPUFilter.h"
+#include "GLShaderParam.h"
 #include "Bitmap.h"
 
 namespace avg {
@@ -37,7 +38,7 @@ public:
     virtual void applyOnGPU(GLTexturePtr pSrcTex);
 
 private:
-    void initShader();
+    IntGLShaderParamPtr m_pTextureParam;
 };
 
 typedef boost::shared_ptr<GPUNullFilter> GPUNullFilterPtr;
