@@ -176,10 +176,10 @@ template <class DataClass>
 LineContribType *
 TwoPassScale<DataClass>::CalcContributions(unsigned uLineSize, unsigned uSrcSize)
 {
-    double dScale = double(uLineSize)/uSrcSize;
-    double dWidth;
-    double dFScale = 1.0;
-    double dFilterWidth = m_ContribDef.GetWidth();
+    float dScale = float(uLineSize)/uSrcSize;
+    float dWidth;
+    float dFScale = 1.0;
+    float dFilterWidth = m_ContribDef.GetWidth();
 
     if (dScale < 1.0) {
         // Minification
@@ -198,7 +198,7 @@ TwoPassScale<DataClass>::CalcContributions(unsigned uLineSize, unsigned uSrcSize
 
     for (unsigned u = 0; u < uLineSize; u++) {
         // Scan through line of contributions
-        double dCenter = (u+0.5)/dScale-0.5;   // Reverse mapping
+        float dCenter = (u+0.5f)/dScale-0.5f;   // Reverse mapping
         // Find the significant edge points that affect the pixel
         int iLeft = std::max (0, (int)floor (dCenter - dWidth));
         int iRight = std::min ((int)ceil (dCenter + dWidth), int(uSrcSize) - 1);
@@ -218,7 +218,7 @@ TwoPassScale<DataClass>::CalcContributions(unsigned uLineSize, unsigned uSrcSize
         for (int iSrc = iLeft; iSrc <= iRight; iSrc++) {
             // Calculate weights
             int CurWeight = int (dFScale * (m_ContribDef.Filter (dFScale * 
-                    (dCenter - (double)iSrc)))*256);
+                    (dCenter - (float)iSrc)))*256);
             res->ContribRow[u].Weights[iSrc-iLeft] = CurWeight;
             dTotalWeight += CurWeight;
         }
@@ -271,15 +271,15 @@ void TwoPassScale<DataClass>::HorizScale(PixelClass * pSrcData, const IntPoint&
         // No scaling required, just copy
         for (int y = 0; y < destSize.y; y++) {
             memcpy(pDest, pSrc, sizeof (PixelClass) * srcSize.x);
-            pSrc += srcStride;
-            pDest += destStride;
+            pSrc = (PixelClass*)((char*)(pSrc)+srcStride);
+            pDest = (PixelClass*)((char*)(pDest)+destStride);
         }
     } else {
         LineContribType * pContrib = CalcContributions(destSize.x, srcSize.x);
         for (int y = 0; y < destSize.y; y++) {
             ScaleRow(pSrc, srcSize.x, pDest, destSize.x, pContrib);
-            pSrc += srcStride;
-            pDest += destStride;
+            pSrc = (PixelClass*)((char*)(pSrc)+srcStride);
+            pDest = (PixelClass*)((char*)(pDest)+destStride);
         }
         FreeContributions(pContrib);  // Free contributions structure
     }
@@ -296,8 +296,8 @@ void TwoPassScale<DataClass>::VertScale(PixelClass *pSrcData, const IntPoint& sr
         // No scaling required, just copy
         for (int y = 0; y < destSize.y; y++) {
             memcpy(pDest, pSrc, sizeof (PixelClass) * srcSize.x);
-            pSrc += srcStride;
-            pDest += destStride;
+            pSrc = (PixelClass*)((char*)(pSrc)+srcStride);
+            pDest = (PixelClass*)((char*)(pDest)+destStride);
         }
     } else {
         LineContribType * pContrib = CalcContributions(destSize.y, srcSize.y);
@@ -306,7 +306,8 @@ void TwoPassScale<DataClass>::VertScale(PixelClass *pSrcData, const IntPoint& sr
             int * pWeights = pContrib->ContribRow[y].Weights;
             int iLeft = pContrib->ContribRow[y].Left;
             int iRight = pContrib->ContribRow[y].Right;
-            PixelClass * pSrcPixelBase = pSrc+iLeft*srcStride;
+            PixelClass* pSrcPixelBase = (PixelClass*)((char*)(pSrc)
+                    + size_t(iLeft)*srcStride);
             for (int x = 0; x < destSize.x; x++) {
                 typename DataClass::_Accumulator a;
                 int * pWeight = pWeights;
@@ -317,12 +318,12 @@ void TwoPassScale<DataClass>::VertScale(PixelClass *pSrcData, const IntPoint& sr
                     // Accumulate weighted effect of each neighboring pixel
                     a.Accumulate(*pWeight, *pSrcPixel);
                     pWeight++;
-                    pSrcPixel += srcStride;
+                    pSrcPixel = (PixelClass*)((char*)(pSrcPixel)+srcStride);
                 }
                 a.Store(pDestPixel);
                 pDestPixel++;
             }
-            pDest += destStride;
+            pDest = (PixelClass*)((char*)(pDest)+destStride);
         }
         FreeContributions(pContrib);     // Free contributions structure
     }
@@ -336,15 +337,15 @@ void TwoPassScale<DataClass>::Scale(PixelClass * pSrcData, const IntPoint& srcSi
     // Allocate temp image
     PixelClass *pTempData = new PixelClass[srcSize.y*dstSize.x];
     IntPoint tempSize(dstSize.x, srcSize.y);
-    int tempStride = dstSize.x;
+    int tempStride = dstSize.x*sizeof(PixelClass);
 
     // Scale source image horizontally into temporary image
-    HorizScale(pSrcData, srcSize, srcStride/sizeof(PixelClass), 
+    HorizScale(pSrcData, srcSize, srcStride, 
             pTempData, tempSize, tempStride);
  
     // Scale temporary image vertically into result image
     VertScale (pTempData, tempSize, tempStride, 
-            pDstData, dstSize, dstStride/sizeof(PixelClass));
+            pDstData, dstSize, dstStride);
     delete [] pTempData;
 }
 
