@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -51,9 +51,11 @@ namespace avg {
 
 using namespace std;
 
-Camera::Camera(PixelFormat camPF, PixelFormat destPF)
+Camera::Camera(PixelFormat camPF, PixelFormat destPF, IntPoint size, float frameRate)
     : m_CamPF(camPF),
-      m_DestPF(destPF)
+      m_DestPF(destPF),
+      m_Size(size),
+      m_FrameRate(frameRate)
 {
 //    cerr << "Camera: " << getPixelFormatString(camPF) << "-->" 
 //        << getPixelFormatString(destPF) << endl;
@@ -74,7 +76,7 @@ PixelFormat Camera::getDestPF() const
     return m_DestPF;
 }
 
-static ProfilingZoneID CameraConvertProfilingZone("Camera format conversion");
+static ProfilingZoneID CameraConvertProfilingZone("Camera format conversion", true);
 
 BitmapPtr Camera::convertCamFrameToDestPF(BitmapPtr pCamBmp)
 {
@@ -85,10 +87,24 @@ BitmapPtr Camera::convertCamFrameToDestPF(BitmapPtr pCamBmp)
         pDestBmp->setPixelFormat(R8G8B8X8);
         FilterFlipRGB().applyInPlace(pDestBmp);
     }
+    if (m_CamPF != R8G8B8 && m_DestPF == R8G8B8X8) {
+        pDestBmp->setPixelFormat(B8G8R8X8);
+        FilterFlipRGB().applyInPlace(pDestBmp);
+    }
 
     return pDestBmp;
 }
 
+IntPoint Camera::getImgSize()
+{
+    return m_Size;
+}
+
+float Camera::getFrameRate() const
+{
+    return m_FrameRate;
+}
+
 PixelFormat Camera::fwBayerStringToPF(unsigned long reg)
 {
     string sBayerFormat((char*)&reg, 4);
@@ -108,6 +124,11 @@ PixelFormat Camera::fwBayerStringToPF(unsigned long reg)
     }
 }
 
+void Camera::setImgSize(const IntPoint& size)
+{
+    m_Size = size;
+}
+
 string cameraFeatureToString(CameraFeature feature)
 {
     switch (feature) {
@@ -137,6 +158,10 @@ string cameraFeatureToString(CameraFeature feature)
             return "temperature";
         case CAM_FEATURE_TRIGGER:
             return "trigger";
+        case CAM_FEATURE_TRIGGER_DELAY:
+            return "trigger delay";
+        case CAM_FEATURE_WHITE_SHADING:
+            return "white shading";
         case CAM_FEATURE_ZOOM:
             return "zoom";
         case CAM_FEATURE_PAN:
@@ -160,7 +185,7 @@ string cameraFeatureToString(CameraFeature feature)
 
 CameraPtr createCamera(const string& sDriver, const string& sDevice, int unit,
         bool bFW800, const IntPoint& captureSize, PixelFormat camPF, PixelFormat destPF, 
-        double frameRate)
+        float frameRate)
 {
     CameraPtr pCamera;
     try {
@@ -182,8 +207,8 @@ CameraPtr createCamera(const string& sDriver, const string& sDevice, int unit,
             pCamera = CameraPtr(new CMUCamera(guid, bFW800, captureSize, camPF, destPF, 
                     frameRate));
 #else
-            guid = 0; // Silence compiler warning
-            AVG_TRACE(Logger::WARNING, "Firewire camera specified, but firewire "
+            (void)guid; // Silence compiler warning
+            AVG_LOG_WARNING("Firewire camera specified, but firewire "
                     "support not compiled in.");
 #endif
         } else if (sDriver == "video4linux") {
@@ -191,7 +216,7 @@ CameraPtr createCamera(const string& sDriver, const string& sDevice, int unit,
             pCamera = CameraPtr(new V4LCamera(sDevice, unit, captureSize, camPF, 
                     destPF, frameRate));
 #else
-            AVG_TRACE(Logger::WARNING, "Video4Linux camera specified, but "
+            AVG_LOG_WARNING("Video4Linux camera specified, but "
                     "Video4Linux support not compiled in.");
 #endif
         } else if (sDriver == "directshow") {
@@ -203,7 +228,7 @@ CameraPtr createCamera(const string& sDriver, const string& sDevice, int unit,
             pCamera = CameraPtr(new DSCamera(sDevice, captureSize, camPF, destPF,
                 frameRate));
 #else
-            AVG_TRACE(Logger::WARNING, "DirectShow camera specified, but "
+            AVG_LOG_WARNING("DirectShow camera specified, but "
                     "DirectShow is only available under windows.");
 #endif
         } else {
@@ -212,7 +237,7 @@ CameraPtr createCamera(const string& sDriver, const string& sDevice, int unit,
         }
     } catch (const Exception& e) {
         if (e.getCode() == AVG_ERR_CAMERA_NONFATAL) {
-            AVG_TRACE(Logger::WARNING, e.getStr());
+            AVG_LOG_WARNING(e.getStr());
         } else {
             throw;
         }
@@ -225,20 +250,52 @@ CameraPtr createCamera(const string& sDriver, const string& sDevice, int unit,
 
 }
 
-void dumpCameras()
+std::vector<CameraInfo> getCamerasInfos()
 {
-#ifdef AVG_ENABLE_1394_2 
-    FWCamera::dumpCameras();
+    std::vector<CameraInfo> camerasInfo;
+    
+#ifdef AVG_ENABLE_1394_2
+    int amountFWCameras = FWCamera::countCameras();
+    for (int i = 0; i < amountFWCameras; i++) {
+        CameraInfo* camInfo = FWCamera::getCameraInfos(i);
+        if (camInfo != NULL) {
+            camInfo->checkAddBayer8();
+            camerasInfo.push_back(*camInfo);
+        }
+    }
 #endif
-#ifdef AVG_ENABLE_CMU1394 
-    CMUCamera::dumpCameras();
+#ifdef AVG_ENABLE_CMU1394
+    int amountCMUCameras = CMUCamera::countCameras();
+    for (int i = 0; i < amountCMUCameras; i++) {
+        CameraInfo* camInfo = CMUCamera::getCameraInfos(i);
+        if (camInfo != NULL) {
+            camInfo->checkAddBayer8();
+            camerasInfo.push_back(*camInfo);
+        }
+    }
 #endif
-#ifdef AVG_ENABLE_V4L2 
-    V4LCamera::dumpCameras();
+#ifdef AVG_ENABLE_DSHOW
+    int amountDSCameras = DSCamera::countCameras();
+    for (int i = 0; i < amountDSCameras; i++) {
+        CameraInfo* camInfo = DSCamera::getCameraInfos(i);
+        if (camInfo != NULL) {
+            camInfo->checkAddBayer8();
+            camerasInfo.push_back(*camInfo);
+        }
+    }
 #endif
-#ifdef AVG_ENABLE_DSHOW 
-    DSCamera::dumpCameras();
+#ifdef AVG_ENABLE_V4L2
+    int amountV4LCameras = V4LCamera::countCameras();
+    for (int i = 0; i < amountV4LCameras; i++) {
+        CameraInfo* camInfo = V4LCamera::getCameraInfos(i);
+        if (camInfo != NULL) {
+            camInfo->checkAddBayer8();
+            camerasInfo.push_back(*camInfo);
+        }
+    }
 #endif
+    return camerasInfo;
 }
 
+
 }
