@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,8 +26,10 @@
 #include "../graphics/Bitmap.h"
 
 #include <boost/shared_ptr.hpp>
+#include "CameraInfo.h"
 
 #include <string>
+#include <list>
 #include <map>
 
 namespace avg {
@@ -46,6 +48,8 @@ enum CameraFeature {
     CAM_FEATURE_FOCUS,
     CAM_FEATURE_TEMPERATURE,
     CAM_FEATURE_TRIGGER,
+    CAM_FEATURE_TRIGGER_DELAY,
+    CAM_FEATURE_WHITE_SHADING,
     CAM_FEATURE_ZOOM,
     CAM_FEATURE_PAN,
     CAM_FEATURE_TILT,
@@ -53,27 +57,28 @@ enum CameraFeature {
     CAM_FEATURE_CAPTURE_SIZE,
     CAM_FEATURE_CAPTURE_QUALITY,
     CAM_FEATURE_CONTRAST,
-    CAM_FEATURE_STROBE_DURATION
+    CAM_FEATURE_STROBE_DURATION,
+    CAM_FEATURE_UNSUPPORTED
 };
 
 class AVG_API Camera
 {
 public:
-    Camera(PixelFormat camPF, PixelFormat destPF);
+    Camera(PixelFormat camPF, PixelFormat destPF, IntPoint size, float frameRate);
     virtual ~Camera() {};
     virtual void startCapture() {};
-
+    
     PixelFormat getCamPF() const;
     void setCamPF(PixelFormat pf);
     PixelFormat getDestPF() const;
     BitmapPtr convertCamFrameToDestPF(BitmapPtr pCamBmp);
 
-    virtual IntPoint getImgSize() = 0;
+    IntPoint getImgSize();
+    float getFrameRate() const;
     virtual BitmapPtr getImage(bool bWait) = 0;
 
     virtual const std::string& getDevice() const = 0; 
     virtual const std::string& getDriverName() const = 0; 
-    virtual double getFrameRate() const = 0;
     
     virtual int getFeature(CameraFeature feature) const = 0;
     virtual void setFeature(CameraFeature feature, int Value, 
@@ -85,13 +90,19 @@ public:
 
 protected:
     PixelFormat fwBayerStringToPF(unsigned long reg);
+    void setImgSize(const IntPoint& size);
 
 private:
     Camera();
     PixelFormat m_CamPF;
     PixelFormat m_DestPF;
+
+    IntPoint m_Size;
+    float m_FrameRate;
 };
 
+
+
 std::string cameraFeatureToString(CameraFeature feature);
 
 typedef boost::shared_ptr<Camera> CameraPtr;
@@ -99,9 +110,9 @@ typedef std::map<CameraFeature, int> FeatureMap;
 
 AVG_API CameraPtr createCamera(const std::string& sDriver, const std::string& sDevice,
         int unit, bool bFW800, const IntPoint& captureSize, PixelFormat camPF, 
-        PixelFormat destPF, double frameRate);
+        PixelFormat destPF, float frameRate);
 
-AVG_API void dumpCameras();
+AVG_API std::vector<CameraInfo> getCamerasInfos();
 
 }
 
