@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -42,11 +42,9 @@ namespace avg {
 using namespace std;
 
 DSCamera::DSCamera(std::string sDevice, IntPoint size, PixelFormat camPF, 
-        PixelFormat destPF, double frameRate)
-    : Camera(camPF, destPF),
+        PixelFormat destPF, float frameRate)
+    : Camera(camPF, destPF, size, frameRate),
       m_sDevice(sDevice),
-      m_Size(size),
-      m_FrameRate(frameRate),
       m_pGraph(0),
       m_pCapture(0),
       m_pCameraPropControl(0)
@@ -135,11 +133,6 @@ void DSCamera::close()
     m_pSampleGrabber->Release();
 }
 
-IntPoint DSCamera::getImgSize()
-{
-    return m_Size;
-}
-
 BitmapPtr DSCamera::getImage(bool bWait)
 {
     BitmapPtr pBmp;
@@ -179,7 +172,7 @@ void DSCamera::setCaptureFormat()
         checkForDShowError(hr, "DSCamera::dumpMediaTypes::GetStreamCaps");
         pvih = (VIDEOINFOHEADER*)(pmtConfig->pbFormat);
         bih = pvih->bmiHeader;
-        double frameRate = double(10000000L/pvih->AvgTimePerFrame);
+        float frameRate = float(10000000L/pvih->AvgTimePerFrame);
         capsPF = mediaSubtypeToPixelFormat(pmtConfig->subtype);
 
         if (capsPF != NO_PIXELFORMAT && bih.biWidth != 0) {
@@ -191,10 +184,10 @@ void DSCamera::setCaptureFormat()
         if (height < 0) {
             height = -height;
         }
-        if (bih.biWidth == m_Size.x && height == m_Size.y && 
+        if (bih.biWidth == getImgSize().x && height == getImgSize().y && 
                 (getCamPF() == capsPF || (getCamPF() == BAYER8_GBRG && capsPF == I8)))
         {
-            if (fabs(m_FrameRate-frameRate) < 0.001) {
+            if (fabs(getFrameRate()-frameRate) < 0.001) {
                 bFormatFound = true;
                 break;
             } else if (!bCloseFormatFound) {
@@ -212,10 +205,10 @@ void DSCamera::setCaptureFormat()
         }
     }
     if (bFormatFound) {
-        AVG_TRACE(Logger::CONFIG, "Camera image format: "
-                << camImageFormatToString(pmtConfig));
+        AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO,
+                "Camera image format: " << camImageFormatToString(pmtConfig));
         int height = ((VIDEOINFOHEADER*)(pmtConfig->pbFormat))->bmiHeader.biHeight;
-//        m_bUpsideDown = (height < 0);
+        m_bUpsideDown = (height > 0);
         hr = pSC->SetFormat(pmtConfig);
         checkForDShowError(hr, "DSCamera::dumpMediaTypes::SetFormat");
         CoTaskMemFree((PVOID)pmtConfig->pbFormat);
@@ -224,22 +217,21 @@ void DSCamera::setCaptureFormat()
         if (bCloseFormatFound) {
             // Set the framerate manually.
             pvih = (VIDEOINFOHEADER*)(pmtCloseConfig->pbFormat);
-            pvih->AvgTimePerFrame = REFERENCE_TIME(10000000/m_FrameRate);
+            pvih->AvgTimePerFrame = REFERENCE_TIME(10000000/getFrameRate());
             int height = pvih->bmiHeader.biHeight;
-//            m_bUpsideDown = (height < 0);
+            m_bUpsideDown = (height > 0);
             hr = pSC->SetFormat(pmtCloseConfig);
             checkForDShowError(hr, "DSCamera::dumpMediaTypes::SetFormat");
-            AVG_TRACE(Logger::CONFIG, "Camera image format: " 
-                    << camImageFormatToString(pmtCloseConfig));
+            AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO,
+                    "Camera image format: " << camImageFormatToString(pmtCloseConfig));
             CoTaskMemFree((PVOID)pmtCloseConfig->pbFormat);
             CoTaskMemFree(pmtCloseConfig);
 
             // TODO: Check if framerate is actually attained.
         } else {
-            AVG_TRACE(Logger::WARNING, 
-                "Possibly incomplete list of camera image formats: ");
+                AVG_LOG_WARNING("Possibly incomplete list of camera image formats: ");
             for (unsigned i = 0; i < sImageFormats.size(); i++) {
-                AVG_TRACE(Logger::WARNING, "  " << sImageFormats[i]);
+                AVG_LOG_WARNING("  " << sImageFormats[i]);
             }
             throw Exception(AVG_ERR_CAMERA_NONFATAL, 
                     "Could not find requested camera image format.");
@@ -260,11 +252,6 @@ const std::string& DSCamera::getDriverName() const
     return sDriverName;
 }
 
-double DSCamera::getFrameRate() const
-{
-    return m_FrameRate;
-}
-
 int DSCamera::getFeature(CameraFeature feature) const
 {
     long prop = getDSFeatureID(feature);
@@ -277,8 +264,8 @@ int DSCamera::getFeature(CameraFeature feature) const
         hr = m_pCameraPropControl->Get(prop, &val, &flags);
     }
     if (!SUCCEEDED(hr)) {
-        AVG_TRACE(Logger::WARNING, "DSCamera::getFeature "
-                + cameraFeatureToString(feature)+" failed.");
+        AVG_LOG_WARNING("DSCamera::getFeature " + cameraFeatureToString(feature)+
+                " failed.");
         return 0;
     }
     return val;
@@ -305,14 +292,13 @@ void DSCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValue
     switch (hr) {
         case E_INVALIDARG:
             // TODO: Throw exception
-            AVG_TRACE(Logger::ERROR, "DSCamera::setFeature(" 
-                    << cameraFeatureToString(feature) << ", " << value << ") failed.");
+            AVG_LOG_ERROR("DSCamera::setFeature(" << cameraFeatureToString(feature) <<
+                    ", " << value << ") failed.");
             break;
         case E_PROP_ID_UNSUPPORTED:  
         case E_PROP_SET_UNSUPPORTED:
-            AVG_TRACE(Logger::ERROR, "DSCamera::setFeature(" 
-                << cameraFeatureToString(feature)
-                << ") failed: Feature not supported by camera.");
+            AVG_LOG_ERROR("DSCamera::setFeature(" << cameraFeatureToString(feature)
+                    << ") failed: Feature not supported by camera.");
             break;
         default:
             checkForDShowError(hr, "DSCamera::setFeature()::Set value");
@@ -321,27 +307,27 @@ void DSCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValue
 
 void DSCamera::setFeatureOneShot(CameraFeature feature)
 {
-    AVG_TRACE(Logger::WARNING, 
+    AVG_LOG_WARNING(
             "OneShot feature setting not implemented for DirectShow camera driver.");
 }
 
 int DSCamera::getWhitebalanceU() const
 {
-    AVG_TRACE(Logger::WARNING, 
+    AVG_LOG_WARNING(
             "Whitebalance not implemented for DirectShow camera driver.");
     return 0;
 }
 
 int DSCamera::getWhitebalanceV() const
 {
-    AVG_TRACE(Logger::WARNING, 
+    AVG_LOG_WARNING(
             "Whitebalance not implemented for DirectShow camera driver.");
     return 0;
 }
 
 void DSCamera::setWhitebalance(int u, int v, bool bIgnoreOldValue)
 {
-    AVG_TRACE(Logger::WARNING, 
+    AVG_LOG_WARNING(
             "Whitebalance not implemented for DirectShow camera driver.");
 }
 
@@ -352,64 +338,240 @@ void DSCamera::onSample(IMediaSample * pSample)
     // Get the current image.
     pSample->GetPointer(&pData);
 
-    int stride = m_Size.x*getBytesPerPixel(getCamPF());
-    Bitmap camBmp(m_Size, getCamPF(), pData, stride, false, "CameraImage");
+    int stride = getImgSize().x*getBytesPerPixel(getCamPF());
+    Bitmap camBmp(getImgSize(), getCamPF(), pData, stride, false, "CameraImage");
     // Copy over to bitmap queue, doing pixel format conversion if necessary.
-    BitmapPtr pDestBmp = BitmapPtr(new Bitmap(m_Size, getDestPF(), 
+    BitmapPtr pDestBmp = BitmapPtr(new Bitmap(getImgSize(), getDestPF(), 
             "ConvertedCameraImage"));
     pDestBmp->copyPixels(camBmp);
-/*
+
     if (m_bUpsideDown) {
         FilterFlip().applyInPlace(pDestBmp);
     }
-*/
+
     m_BitmapQ.push(pDestBmp);
 }
 
-void DSCamera::dumpCameras()
+int DSCamera::countCameras()
 {
+    int count = 0;
     HRESULT hr = S_OK;
-    // TODO: Check if the threading model is ok.
     hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
-    checkForDShowError(hr, "DSCamera::dumpCameras()::CoInitializeEx");
+    checkForDShowError(hr, "DSCamera::countCameras()::CoInitializeEx");
 
+    ICreateDevEnum *pDevEnum =NULL;
+    hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, 
+            IID_ICreateDevEnum, (void **) &pDevEnum);
+    checkForDShowError(hr, "DSCamera::countCameras()::CreateDevEnum");
+
+    IEnumMoniker *pClassEnum = NULL;
+    hr = pDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
+    checkForDShowError(hr, "DSCamera::countCameras()::CreateClassEnumerator");
+
+    if (pClassEnum == NULL) {
+        pDevEnum->Release();
+        return count;
+    }
+    IMoniker* pMoniker = NULL;
+    while (pClassEnum->Next(1, &pMoniker, NULL) == S_OK) {
+        count += 1;
+    }
+    pMoniker->Release();
+    pClassEnum->Release();
+    pDevEnum->Release();
+    return count;
+}
+
+CameraInfo* DSCamera::getCameraInfos(int deviceNumber)
+{
+#ifdef AVG_ENABLE_DSHOW
+    HRESULT hr = S_OK;
+    // Create apartment for Thread
+    hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
+    checkForDShowError(hr, "DSCamera::getCameraInfos()::CoInitializeEx");
     // Create the system device enumerator
     ICreateDevEnum *pDevEnum =NULL;
     hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, 
             IID_ICreateDevEnum, (void **) &pDevEnum);
-    checkForDShowError(hr, "DSCamera::dumpCameras()::CreateDevEnum");
-
+    checkForDShowError(hr, "DSCamera::getCameraInfos()::CreateDevEnum");
     // Create an enumerator for the video capture devices
     IEnumMoniker *pClassEnum = NULL;
     hr = pDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
-    checkForDShowError(hr, "DSCamera::dumpCameras()::CreateClassEnumerator");
-
+    checkForDShowError(hr, "DSCamera::getCameraInfos()::CreateClassEnumerator");
     if (pClassEnum == NULL) {
+        pClassEnum->Release();
+        pDevEnum->Release();
+        return NULL;
+    }
+    IMoniker* pMoniker = NULL;
+    pClassEnum->Skip(deviceNumber);
+    hr = pClassEnum->Next(1, &pMoniker, NULL);
+    if (hr != S_OK) {
+        pClassEnum->Release();
+        pDevEnum->Release();
+        pMoniker->Release();
+        return NULL;
+    }
+    IPropertyBag* pPropBag;
+    hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void**)(&pPropBag));
+    checkForDShowError(hr, "DSCamera::getCameraInfos()::BindToStorage");
+
+    std::string deviceID = getStringProp(pPropBag, L"DevicePath");
+    if (deviceID.empty()) {
+        deviceID = getStringProp(pPropBag, L"Description");
+        if (deviceID.empty()) {
+            deviceID = getStringProp(pPropBag, L"FriendlyName");
+        }
+    }
+    CameraInfo* pCamInfo = new CameraInfo("DirectShow", deviceID);
+
+    getCameraImageFormats(pMoniker, pCamInfo);
+    getCameraControls(pMoniker, pCamInfo);
+            
+    pPropBag->Release();
+    pMoniker->Release();
+    pClassEnum->Release();
+    pDevEnum->Release();
+    return pCamInfo;
+#endif
+    return NULL;
+}
+
+void DSCamera::getCameraImageFormats(IMoniker* pMoniker, CameraInfo* pCamInfo)
+{
+    HRESULT hr = S_OK;
+    IAMStreamConfig* pSC;
+    ICaptureGraphBuilder2* pCapture;
+    IBaseFilter* pSrcFilter;
+    // locates the object identified by pMoniker and 
+    // returns a pointer to its filter interface
+    hr = pMoniker->BindToObject(0,0,IID_IBaseFilter, (void**) &pSrcFilter);
+    checkForDShowError(hr, "DSCamera::getImageFormats()::BindToObject");
+    if (pSrcFilter == NULL) {
         return;
     }
+    // Creates an uninitialized instance and returns a pointer to 
+    // the IID_ICaptureGraphBuilder2 interface
+    hr = CoCreateInstance (CLSID_CaptureGraphBuilder2 , NULL, CLSCTX_INPROC,
+            IID_ICaptureGraphBuilder2, (void **) &pCapture);
+    checkForDShowError(hr, "DSCamera::getImageFormats()::CaptureGraphBuilder2");
+    // searches the graph for a IID_IAMStreamConfig interface, returns a pointer
+    hr = pCapture->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, 
+            pSrcFilter, IID_IAMStreamConfig, (void **)&pSC);
+    checkForDShowError(hr, "DSCamera::getImageFormats()::FindInterface");
+    int numCaps = 0;
+    int capsSize = 0;
+    hr = pSC->GetNumberOfCapabilities(&numCaps, &capsSize);
+    checkForDShowError(hr, "DSCamera::getImageFormats()::GetNumberOfCapabilities");
+    AM_MEDIA_TYPE* pmtConfig;
+    vector<string> sImageFormats;
+    VIDEOINFOHEADER* pvih;
+    BITMAPINFOHEADER bih;
+    PixelFormat capsPF;
+    for (int i = 0; i < numCaps; i++) {
+        VIDEO_STREAM_CONFIG_CAPS scc;
+        hr = pSC->GetStreamCaps(i, &pmtConfig, (BYTE*)&scc);
+        checkForDShowError(hr, "DSCamera::getImageFormats()::GetStreamCaps");
+        pvih = (VIDEOINFOHEADER*)(pmtConfig->pbFormat);
+        bih = pvih->bmiHeader;
+        capsPF = mediaSubtypeToPixelFormat(pmtConfig->subtype);
 
-    IMoniker* pMoniker = NULL;
-    bool bFirst = true;
-    while (pClassEnum->Next(1, &pMoniker, NULL) == S_OK) {
-        if (bFirst) {
-            cerr << endl;
-            cerr << "DirectShow cameras: " << endl;
-            bFirst = false;
+        if (capsPF != NO_PIXELFORMAT && bih.biWidth != 0) {
+            IntPoint size;
+            if (bih.biHeight >= 0) {
+                size = IntPoint(bih.biWidth, bih.biHeight);
+            } else {
+                size = IntPoint(bih.biWidth, -bih.biHeight);
+            }
+
+            std::vector<float> framerates;
+            float minFramerate = (float)(10000000 / scc.MinFrameInterval);
+            float maxFramerate = (float)(10000000 / scc.MaxFrameInterval);
+            float averageFramerate = (float)(10000000 / pvih->AvgTimePerFrame);
+            if (maxFramerate != 0.) {
+                framerates.push_back(maxFramerate);
+            }
+            if (averageFramerate != maxFramerate && averageFramerate != minFramerate) {
+                framerates.push_back(averageFramerate);
+            }
+            if (minFramerate != 0.  && minFramerate != maxFramerate) {
+                framerates.push_back(minFramerate);
+            }
+            CameraImageFormat imageFormat = CameraImageFormat(size, capsPF, framerates);
+            pCamInfo->addImageFormat(imageFormat);
         }
-        IPropertyBag* pPropBag;
-        hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void**)(&pPropBag));
-        checkForDShowError(hr, "DSCamera::dumpCameras()::BindToStorage");
-        cerr << "  ----------------------------" << endl;
-        cerr << "  Name: " << getStringProp(pPropBag, L"FriendlyName") << endl;
-        cerr << "  Description: " << getStringProp(pPropBag, L"Description") << endl;
-        cerr << "  Device Path: " << getStringProp(pPropBag, L"DevicePath") << endl;
-        pPropBag->Release();
     }
-    pMoniker->Release();
-    pDevEnum->Release();
-    pClassEnum->Release();
+    pCapture->Release();
+    pSrcFilter->Release();
 }
 
+void DSCamera::getCameraControls(IMoniker* pMoniker, CameraInfo* pCamInfo)
+{
+    HRESULT hr = S_OK;
+    IBaseFilter * pSrcFilter;
+    // locates the object identified by pMoniker and 
+    // returns a pointer to its filter interface
+    hr = pMoniker->BindToObject(0,0,IID_IBaseFilter, (void**) &pSrcFilter);
+    checkForDShowError(hr, "DSCamera::dumpImageFormats()::BindToObject");
+    if (pSrcFilter == NULL) {
+        return;
+    }
+    IAMCameraControl* pAMCameraControl;
+    pSrcFilter->QueryInterface(IID_IAMCameraControl, 
+                (void **)&pAMCameraControl);
+    if (pAMCameraControl != NULL) {
+        // DirectShow has 7 supported CameraControlProperties
+        for (int indexControl = 0; indexControl <= 6; indexControl++) {
+            long value = -999;
+            long flags = -999;
+            pAMCameraControl->Get((CameraControlProperty)indexControl, &value, &flags);
+            long min = -999;
+            long max = -999;
+            long delta = -999;
+            long defaultValue = -999;
+            flags = -999;
+            pAMCameraControl->GetRange((CameraControlProperty)indexControl, &min, &max,
+                    &delta, &defaultValue, &flags);
+
+            CameraFeature feature = getCameraFeatureID_CCP((CameraControlProperty)indexControl);
+            if (min != -999 && max != -999 && defaultValue != -999
+                        && feature != CAM_FEATURE_UNSUPPORTED) {
+                std::string featureName = cameraFeatureToString(feature);
+                CameraControl control = CameraControl(featureName,min,max,defaultValue);
+                pCamInfo->addControl(control);
+            }
+        }
+    }
+    IAMVideoProcAmp* pCameraPropControl;
+    pSrcFilter->QueryInterface(IID_IAMVideoProcAmp, 
+                (void **)&pCameraPropControl);
+    if (pCameraPropControl != NULL) {
+        // DirectShow has 10 supported VideoProcAmpProperties
+        for (int indexPropControl = 0; indexPropControl <= 9; indexPropControl++) {
+            long value = -999;
+            long flags = -999;
+            pCameraPropControl->Get((VideoProcAmpProperty)indexPropControl, &value,
+                    &flags);
+            long min = -999;
+            long max = -999;
+            long delta = -999;
+            long defaultValue = -999;
+            flags = -999;
+            pCameraPropControl->GetRange((VideoProcAmpProperty)indexPropControl, &min,
+                    &max, &delta, &defaultValue, &flags);
+
+            CameraFeature feature = getCameraFeatureID_VPAP((VideoProcAmpProperty)indexPropControl);
+            if (min != -999 && max != -999 && defaultValue != -999
+                    && feature != CAM_FEATURE_UNSUPPORTED) {
+                std::string featureName = cameraFeatureToString(feature);
+                CameraControl control = CameraControl(featureName,min,max,defaultValue);
+                pCamInfo->addControl(control);
+            }
+        }
+    }
+}
+
+
 void DSCamera::initGraphBuilder()
 {
     HRESULT hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
@@ -477,7 +639,7 @@ void DSCamera::findCaptureDevice(IBaseFilter ** ppSrcFilter)
     if (!bFound) {
         pClassEnum->Reset();
         if (pClassEnum->Next(1, &pMoniker, NULL) == S_OK) {
-            AVG_TRACE(Logger::WARNING, string("Camera ") + m_sDevice
+            AVG_LOG_WARNING(string("Camera ") + m_sDevice
                     + " not found. Using first camera.");
             bFound = true;
             IPropertyBag *pPropBag;
