@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -37,16 +37,14 @@ namespace avg {
 
 using namespace std;
 
-FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size, 
-        PixelFormat camPF, PixelFormat destPF, double frameRate)
-    : Camera(camPF, destPF),
-      m_Size(size),
-      m_FrameRate(frameRate),
+FWCamera::FWCamera(unsigned long long guid, int unit, bool bFW800, IntPoint size, 
+        PixelFormat camPF, PixelFormat destPF, float frameRate)
+    : Camera(camPF, destPF, size, frameRate),
       m_WhitebalanceU(-1),
       m_WhitebalanceV(-1)
 {
 #ifdef AVG_ENABLE_1394_2
-    m_FrameRateConstant = getFrameRateConst(m_FrameRate);
+    m_FrameRateConstant = getFrameRateConst(getFrameRate());
     if (camPF == I16) {
         throw Exception(AVG_ERR_CAMERA_NONFATAL, 
                 "I16 pixel format is not supported for firewire cameras.");
@@ -62,13 +60,12 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
     int err = dc1394_camera_enumerate(m_pDC1394, &pCameraList);
 
     if (err != DC1394_SUCCESS) {
-        AVG_TRACE(Logger::ERROR, "Unable to look for cameras");
+        AVG_LOG_ERROR("Unable to look for cameras");
 #ifdef linux
-        AVG_TRACE(Logger::ERROR, "Please check");
-        AVG_TRACE(Logger::ERROR,
-              "  - if the kernel modules `ieee1394',`raw1394' and `ohci1394' are loaded");
-        AVG_TRACE(Logger::ERROR,
-              "  - if you have read/write access to /dev/raw1394.");
+        AVG_LOG_ERROR("Please check");
+        AVG_LOG_ERROR("  - if the kernel modules `ieee1394',`raw1394' and \
+                `ohci1394' are loaded");
+        AVG_LOG_ERROR("  - if you have read/write access to /dev/raw1394.");
 #endif
         dc1394_free(m_pDC1394);
         throw Exception(AVG_ERR_CAMERA_NONFATAL,"Firewire failure");
@@ -87,7 +84,7 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
             }       
         }
         if (id_to_use == -1) {
-            AVG_TRACE(Logger::WARNING, "Firewire GUID=" << hex << guid << dec
+            AVG_LOG_WARNING("Firewire GUID=" << hex << guid << dec
                     << " requested but not found on bus. Using first camera");
             id_to_use = 0;
         }
@@ -129,14 +126,14 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
         }
     }
     if (!bFrameRateSupported) {
-        AVG_TRACE(Logger::ERROR, "Camera does not support framerate " << m_FrameRate 
+        AVG_LOG_ERROR("Camera does not support framerate " << getFrameRate() 
                 << " in the current video mode.");
         dc1394_capture_stop(m_pCamera);
         dc1394_video_set_transmission(m_pCamera, DC1394_OFF);
         dc1394_camera_free(m_pCamera);
         dc1394_free(m_pDC1394);
         throw Exception(AVG_ERR_CAMERA_NONFATAL, 
-                string("Camera does not support framerate ")+toString(m_FrameRate)+
+                string("Camera does not support framerate ")+toString(getFrameRate())+
                 " in the current video mode.");
     }
 
@@ -145,10 +142,9 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
 
     err = dc1394_capture_setup(m_pCamera,8, DC1394_CAPTURE_FLAGS_DEFAULT);
     if (err != DC1394_SUCCESS) {
-        AVG_TRACE(Logger::ERROR, "Unable to setup camera. Make sure that");
-        AVG_TRACE(Logger::ERROR, "video mode and framerate (" <<
-                m_FrameRate << ") are");
-        AVG_TRACE(Logger::ERROR, "supported by your camera.");
+        AVG_LOG_ERROR("Unable to setup camera. Make sure that");
+        AVG_LOG_ERROR("video mode and framerate (" << getFrameRate() << ") are");
+        AVG_LOG_ERROR("supported by your camera.");
         dc1394_capture_stop(m_pCamera);
         dc1394_video_set_transmission(m_pCamera, DC1394_OFF);
         dc1394_camera_free(m_pCamera);
@@ -168,7 +164,7 @@ FWCamera::~FWCamera()
     dc1394_camera_free(m_pCamera);
     dc1394_free(m_pDC1394);
 #endif
-    AVG_TRACE(Logger::CONFIG, "Firewire camera closed.");
+    AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO, "Firewire camera closed.");
 }
 
 void FWCamera::startCapture()
@@ -197,7 +193,7 @@ void FWCamera::startCapture()
             DC1394_FEATURE_MODE_MANUAL);
     dc1394_feature_set_power(m_pCamera, DC1394_FEATURE_EXPOSURE, DC1394_OFF);
 
-    AVG_TRACE(Logger::CONFIG, "Firewire camera opened.");
+    AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO, "Firewire camera opened.");
     for (FeatureMap::iterator it=m_Features.begin(); it != m_Features.end(); it++) {
         setFeature(it->first, it->second, true);
     }
@@ -205,11 +201,11 @@ void FWCamera::startCapture()
     
     if (getCamPF() == BAYER8) {
         if (strcmp(m_pCamera->model, "DFx 31BF03") == 0) {
-            AVG_TRACE(Logger::CONFIG,
+            AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO,
                     "Applying bayer pattern fixup for IS DFx31BF03 camera");
             setCamPF(BAYER8_GRBG);
         } else if (strcmp(m_pCamera->vendor, "Point Grey Research") == 0) {
-            AVG_TRACE(Logger::CONFIG,
+            AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO,
                     "Applying bayer pattern fixup for PointGrey cameras");
             enablePtGreyBayer();
         }
@@ -217,18 +213,6 @@ void FWCamera::startCapture()
 #endif
 }
 
-IntPoint FWCamera::getImgSize()
-{
-#ifdef AVG_ENABLE_1394_2
-    return m_Size;
-#else
-    return IntPoint(0, 0);
-#endif
-}
-
-
-static ProfilingZoneID CameraConvertProfilingZone("FW Camera format conversion");
-
 BitmapPtr FWCamera::getImage(bool bWait)
 {
 #ifdef AVG_ENABLE_1394_2
@@ -248,12 +232,12 @@ BitmapPtr FWCamera::getImage(bool bWait)
     if (bGotFrame) {
         int lineLen;
         if (getCamPF() == YCbCr411) {
-            lineLen = m_Size.x*1.5;
+            lineLen = getImgSize().x*1.5;
         } else {
-            lineLen = m_Size.x*getBytesPerPixel(getCamPF());
+            lineLen = getImgSize().x*getBytesPerPixel(getCamPF());
         }
-        BitmapPtr pCamBmp(new Bitmap(m_Size, getCamPF(), pCaptureBuffer, lineLen, false,
-                "TempCameraBmp"));
+        BitmapPtr pCamBmp(new Bitmap(getImgSize(), getCamPF(), pCaptureBuffer, lineLen,
+                false, "TempCameraBmp"));
         BitmapPtr pDestBmp = convertCamFrameToDestPF(pCamBmp);
 //        cerr << "CamBmp: " << pCamBmp->getPixelFormat() << ", DestBmp: " 
 //                << pDestBmp->getPixelFormat() << endl;
@@ -290,11 +274,6 @@ const std::string& FWCamera::getDriverName() const
     return sDriverName;
 }
 
-double FWCamera::getFrameRate() const
-{
-    return m_FrameRate;
-}
-
 int FWCamera::getFeature(CameraFeature feature) const
 {
 #ifdef AVG_ENABLE_1394_2
@@ -336,8 +315,7 @@ void FWCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValue
                 try {
                     setStrobeDuration(value);
                 } catch (Exception& e) {
-                    AVG_TRACE(Logger::WARNING, 
-                            string("Camera: Setting strobe duration failed. ") + 
+                    AVG_LOG_WARNING(string("Camera: Setting strobe duration failed. ") + 
                             e.getStr());
                 }
             } else {
@@ -357,7 +335,7 @@ void FWCamera::setFeatureOneShot(CameraFeature feature)
     dc1394error_t err = dc1394_feature_set_mode(m_pCamera, featureID, 
             DC1394_FEATURE_MODE_ONE_PUSH_AUTO);
     if (err != DC1394_SUCCESS) {
-        AVG_TRACE(Logger::WARNING, "Camera: Unable to set one-shot for " 
+        AVG_LOG_WARNING("Camera: Unable to set one-shot for " 
                 << cameraFeatureToString(feature) << ". Error was " << err);
     }
 #endif
@@ -396,8 +374,7 @@ void FWCamera::setWhitebalance(int u, int v, bool bIgnoreOldValue)
                 err = dc1394_feature_whitebalance_set_value(m_pCamera, u, v);
             }
             if (err != DC1394_SUCCESS) {
-                AVG_TRACE(Logger::WARNING,
-                        "Camera: Unable to set whitebalance. Error was " << err);
+                AVG_LOG_WARNING("Camera: Unable to set whitebalance. Error was " << err);
             }
         }
     }
@@ -417,8 +394,7 @@ void FWCamera::setFeature(dc1394feature_t feature, int value)
         err = dc1394_feature_set_value(m_pCamera, feature, value);
     }
     if (err != DC1394_SUCCESS) {
-        AVG_TRACE(Logger::WARNING, "Camera: Unable to set " << feature << 
-                ". Error was " << err);
+        AVG_LOG_WARNING("Camera: Unable to set " << feature << ". Error was " << err);
     }
 /*
     dc1394feature_info_t featureInfo;
@@ -450,16 +426,17 @@ void FWCamera::setStrobeDuration(int microsecs)
         } else {
             // Wierd calculations: IIDC register values for time are non-linear. Translate
             // the method parameter in microseconds to appropriate register values.
-            double targetMillisecs = microsecs/1000.;
-            const double realTimes[] = {1,2,4,6,8,12,16,24,32,48,63.93};
+            float targetMillisecs = microsecs/1000.f;
+            const float realTimes[] = {1,2,4,6,8,12,16,24,32,48,63.93};
             const uint32_t regValues[] = 
                 {0x400, 0x600, 0x800, 0x900, 0xA00, 0xB00, 0xC00, 0xD00, 
                  0xE00, 0xF00, 0xFFF};
             int len = sizeof(regValues)/sizeof(*regValues);
             AVG_ASSERT(len == sizeof(realTimes)/sizeof(*realTimes));
             int i;
-            for (i = 1; realTimes[i] < targetMillisecs; ++i); 
-            double ratio = (targetMillisecs-realTimes[i])/(realTimes[i-1]-realTimes[i]);
+            for (i = 1; realTimes[i] < targetMillisecs; ++i) {
+            } 
+            float ratio = (targetMillisecs-realTimes[i])/(realTimes[i-1]-realTimes[i]);
             durationRegValue = ratio*regValues[i-1]+(1-ratio)*regValues[i];
         } 
 
@@ -479,8 +456,7 @@ void FWCamera::getWhitebalance(int* pU, int* pV) const
     dc1394error_t err = dc1394_feature_whitebalance_get_value(m_pCamera, 
             (uint32_t*)pU, (uint32_t*)pV);
     if (err != DC1394_SUCCESS) {
-        AVG_TRACE(Logger::WARNING,
-                "Camera: Unable to get whitebalance setting. Error was " << err);
+        AVG_LOG_WARNING("Camera: Unable to get whitebalance setting. Error was " << err);
     }
 #endif
 }
@@ -508,45 +484,164 @@ void FWCamera::enablePtGreyBayer()
 #endif
 }
 
-void FWCamera::dumpCameras()
+int FWCamera::countCameras()
 {
 #ifdef AVG_ENABLE_1394_2
     dc1394_t* pDC1394 = dc1394_new();
     if (pDC1394 == 0) {
-        return;
+        return 0;
     }
     dc1394camera_list_t * pCameraList;
     int err=dc1394_camera_enumerate(pDC1394, &pCameraList);
     if (err == DC1394_SUCCESS) {
-        if (pCameraList->num != 0) {
-            cerr << "Firewire cameras: " << endl;
-            for (unsigned i=0; i<pCameraList->num;++i) {
-                dc1394camera_id_t id = pCameraList->ids[i];
-                dc1394camera_t * pCamera = dc1394_camera_new_unit(pDC1394, id.guid, 
-                        id.unit);
-                if (pCamera) {
-                    dc1394_camera_print_info(pCamera, stderr);
-                    dumpCameraInfo(pCamera);
-                    dc1394_camera_free(pCamera);
-                    cerr << endl;
-                }
-            }
-        }
-        dc1394_camera_free_list(pCameraList);
+        int numCameras = pCameraList->num;
+        return numCameras;
     }
-    dc1394_free(pDC1394);
 #endif
+    return 0;
+}
+
+CameraInfo* FWCamera::getCameraInfos(int deviceNumber)
+{
+#ifdef AVG_ENABLE_1394_2
+    dc1394_t* pDC1394 = dc1394_new();
+    if (pDC1394 == 0) {
+        AVG_ASSERT(false);
+        return NULL;
+    }
+    dc1394camera_list_t * pCameraList;
+    int err=dc1394_camera_enumerate(pDC1394, &pCameraList);
+    if (err != DC1394_SUCCESS) {
+        AVG_ASSERT(false);
+        return NULL;
+    }
+    if (pCameraList->num != 0) {
+        dc1394camera_id_t id = pCameraList->ids[deviceNumber];
+        dc1394camera_t * pCamera = dc1394_camera_new_unit(pDC1394, id.guid,
+                id.unit);
+        if (pCamera) {
+            stringstream deviceID;
+            deviceID << hex << id.guid;//pCamera->guid;
+            CameraInfo* camInfo = new CameraInfo("Firewire", deviceID.str());
+
+            getCameraControls(pCamera, camInfo);
+            getCameraImageFormats(pCamera, camInfo);
+
+            dc1394_camera_free(pCamera);
+            dc1394_camera_free_list(pCameraList);
+            dc1394_free(pDC1394);
+            return camInfo;
+        }
+    }
+#endif
+    return NULL;
 }
 
 #ifdef AVG_ENABLE_1394_2
-void FWCamera::dumpCameraInfo(dc1394camera_t * pCamera)
+void FWCamera::getCameraImageFormats(dc1394camera_t* pCamera, CameraInfo* camInfo)
 {
-    dc1394error_t err;
-    dc1394featureset_t FeatureSet;
-    err = dc1394_feature_get_all(pCamera, &FeatureSet);
-    AVG_ASSERT(err == DC1394_SUCCESS);
-    // TODO: do this using AVG_TRACE
-    dc1394_feature_print_all(&FeatureSet, stderr);
+    dc1394video_modes_t videoModes;
+    dc1394framerates_t framerates;
+    dc1394error_t err = dc1394_video_get_supported_modes(pCamera, &videoModes);
+    if (err != DC1394_SUCCESS) {
+        AVG_ASSERT(false);
+        return;
+    }
+    for (unsigned i = 0; i < videoModes.num; i++) {
+        //Covers only libavg supported formats, other capabilities are ignored
+        if (videoModes.modes[i] >= DC1394_VIDEO_MODE_320x240_YUV422
+                && videoModes.modes[i] <= DC1394_VIDEO_MODE_1600x1200_MONO16){
+            PixelFormat pixFormat = getPFFromVideoMode(videoModes.modes[i]);
+            IntPoint size = getFrameSizeFromVideoMode(videoModes.modes[i]);
+            FrameratesVector framerateList;
+            err = dc1394_video_get_supported_framerates(pCamera, videoModes.modes[i],
+                    &framerates);
+            if (err != DC1394_SUCCESS) {
+                AVG_LOG_WARNING("Camera: No framerates. Error was: " << err);
+            } else {
+                for (unsigned j = 0; j < framerates.num; j++)
+                {
+                    float rate = framerateToFloat(framerates.framerates[j]);
+                    framerateList.push_back(rate);
+                }
+            }
+            CameraImageFormat format = CameraImageFormat(size,pixFormat,framerateList);
+            camInfo->addImageFormat(format);
+        }
+    }
+}
+
+void FWCamera::getCameraControls(dc1394camera_t* pCamera, CameraInfo* camInfo)
+{
+    dc1394featureset_t featureSet;
+    int err = dc1394_feature_get_all(pCamera, &featureSet);
+    if (err != DC1394_SUCCESS) {
+        AVG_ASSERT(false);
+        return;
+    }
+
+    for (int i = DC1394_FEATURE_MIN; i <= DC1394_FEATURE_MAX; i++) {
+        dc1394feature_info_t featureInfo = featureSet.feature[i - DC1394_FEATURE_MIN];
+
+        dc1394bool_t bool_t;
+        dc1394_feature_is_present(pCamera,featureInfo.id, &bool_t);
+        if (bool_t != DC1394_TRUE) {
+            continue;
+        }
+
+        uint32_t min = -1;
+        uint32_t max = -1;
+        uint32_t actValue = -1;
+
+        //TODO: 428 (TRIGGER) doesnt have min max
+        err = dc1394_feature_get_boundaries(pCamera, featureInfo.id, &min, &max);
+        if (err != DC1394_SUCCESS) {
+            continue;
+        }
+
+        switch(featureInfo.id) {
+            case DC1394_FEATURE_TEMPERATURE: {
+                uint32_t targetTemp = -1;
+                uint32_t currentTemp = -1;
+                err = dc1394_feature_temperature_get_value(pCamera,&targetTemp,&currentTemp);
+                if (err != DC1394_SUCCESS) {
+                    continue;
+                }
+                actValue = currentTemp;
+                break;
+            }
+        //TODO: Think about a way to get this information into CameraInfo
+            case DC1394_FEATURE_WHITE_BALANCE: {
+                uint32_t ubValue = -1;
+                uint32_t vrValue = -1;
+                err = dc1394_feature_whitebalance_get_value(pCamera,&ubValue,&vrValue);
+                if (err != DC1394_SUCCESS) {
+                    continue;
+                }
+                //actValue = ubValue; //vrValue;
+                //cout <<"UBlue Value: " << ubValue << " VRed Value: " << vrValue << endl;
+                break;
+            }
+            default: {
+                err = dc1394_feature_get_value(pCamera,featureInfo.id, &actValue);
+                if (err != DC1394_SUCCESS) {
+                    continue;
+                }
+                break;
+            }
+        }
+        CameraFeature enumFeature = featureIDToEnum(featureInfo.id);
+        if (enumFeature == CAM_FEATURE_UNSUPPORTED) {
+            continue;
+        }
+        std::string controlName = cameraFeatureToString(enumFeature);
+
+        CameraControl control = CameraControl(controlName,
+                (int) min,
+                (int) max,
+                (int) actValue ); //TODO: isnt really a default value!?
+        camInfo->addControl(control);
+    }
 }
 #endif
 
