@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@
 
 #include "../avgconfigwrapper.h"
 
-#include "../base/Point.h"
+#include "../base/GLMHelper.h"
 
 #include <dc1394/control.h>
 
@@ -36,8 +36,12 @@
 namespace avg {
 
 dc1394video_mode_t getCamMode(IntPoint size, PixelFormat pf);
-dc1394framerate_t getFrameRateConst(double frameRate);
+dc1394framerate_t getFrameRateConst(float frameRate);
+CameraFeature featureIDToEnum(dc1394feature_t feature);
 dc1394feature_t getFeatureID(CameraFeature feature);
+IntPoint getFrameSizeFromVideoMode(dc1394video_mode_t mode);
+PixelFormat getPFFromVideoMode(dc1394video_mode_t mode);
+float framerateToFloat(dc1394framerate_t framerate);
 
 }
 
