@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@ Run::Run(int row, int startCol, int endCol)
     m_Row = row;
     m_StartCol = startCol;
     m_EndCol = endCol;
-    m_Center = DPoint((m_StartCol + m_EndCol-1)/2., m_Row);
+    m_Center = glm::vec2((m_StartCol + m_EndCol-1)/2., m_Row);
 }
  
 }
