@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -70,15 +70,17 @@ void TrackerConfig::loadConfigFile(const string& sFilename)
     string sDTDFName = "trackerconfig.dtd";
     dtd = xmlParseDTD(NULL, (const xmlChar*) sDTDFName.c_str());
     if (!dtd) {
-        AVG_TRACE(Logger::WARNING, 
-                "DTD not found at " << sDTDFName 
+        AVG_LOG_WARNING("DTD not found at " << sDTDFName 
                 << ". Not validating trackerconfig files.");
     }
 
-    m_Doc = xmlParseFile(sFilename.c_str());
+    // xmlParseFile crashes for some reason under Lion.
+    string sFileContents;
+    readWholeFile(sFilename, sFileContents);
+    m_Doc = xmlParseMemory(sFileContents.c_str(), sFileContents.length());
     if (!m_Doc) {
-        AVG_TRACE(Logger::ERROR, "Could not open tracker config file " 
-                << sFilename << ". Using defaults which will probably not work.");
+        AVG_LOG_ERROR("Could not open tracker config file " << sFilename <<
+                ". Using defaults which will probably not work.");
         return;
     }
 
@@ -95,7 +97,8 @@ void TrackerConfig::loadConfigFile(const string& sFilename)
     xmlFreeDtd(dtd);
     m_sFilename = sFilename;
 
-    AVG_TRACE(Logger::CONFIG, "Reading Tracker config file from " << sFilename);
+    AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO,
+            "Reading Tracker config file from " << sFilename);
 }
 
 void TrackerConfig::load()
@@ -117,13 +120,13 @@ xmlXPathObjectPtr TrackerConfig::findConfigNodes(const string& sXPathExpr) const
 
     xpCtx = xmlXPathNewContext(m_Doc);
     if(!xpCtx) {
-        AVG_TRACE(Logger::ERROR, "Unable to create new XPath context");
+        AVG_LOG_ERROR("Unable to create new XPath context");
         return NULL;
     }
 
     xpElement = xmlXPathEvalExpression(BAD_CAST sFullPath.c_str(), xpCtx);
     if(!xpElement) {
-        AVG_TRACE(Logger::ERROR, "Unable to evaluate XPath expression '"
+        AVG_LOG_ERROR("Unable to evaluate XPath expression '"
             << sFullPath << "'");
         xmlXPathFreeContext(xpCtx);
         return NULL;
@@ -163,7 +166,7 @@ string TrackerConfig::getParam(const string& sXPathExpr) const
         throw (Exception(AVG_ERR_OPTION_UNKNOWN, 
                     string("getParam(): cannot find requested element ")+sXPathExpr));
     } else if (nodes->nodeNr > 1) {
-        AVG_TRACE(Logger::WARNING,
+        AVG_LOG_WARNING(
             "getParam(): expression selects more than one node. Returning the first.");
     }
 
@@ -186,23 +189,23 @@ int TrackerConfig::getIntParam(const std::string& sXPathExpr) const
     return stringToInt(getParam(sXPathExpr));
 }
 
-double TrackerConfig::getDoubleParam(const std::string& sXPathExpr) const
+float TrackerConfig::getFloatParam(const std::string& sXPathExpr) const
 {
-    return stringToDouble(getParam(sXPathExpr));
+    return stringToFloat(getParam(sXPathExpr));
 }
 
-DPoint TrackerConfig::getPointParam(const std::string& sXPathExpr) const
+glm::vec2 TrackerConfig::getPointParam(const std::string& sXPathExpr) const
 {
-    return DPoint(getDoubleParam(sXPathExpr+"@x"), getDoubleParam(sXPathExpr+"@y"));
+    return glm::vec2(getFloatParam(sXPathExpr+"@x"), getFloatParam(sXPathExpr+"@y"));
 }
 
-DRect TrackerConfig::getRectParam(const std::string& sXPathExpr) const
+FRect TrackerConfig::getRectParam(const std::string& sXPathExpr) const
 {
-     DPoint pos1 = DPoint(getDoubleParam(sXPathExpr+"@x1"), 
-            getDoubleParam(sXPathExpr+"@y1"));
-     DPoint pos2 = DPoint(getDoubleParam(sXPathExpr+"@x2"),
-            getDoubleParam(sXPathExpr+"@y2"));
-     return DRect(pos1, pos2);
+     glm::vec2 pos1 = glm::vec2(getFloatParam(sXPathExpr+"@x1"), 
+            getFloatParam(sXPathExpr+"@y1"));
+     glm::vec2 pos2 = glm::vec2(getFloatParam(sXPathExpr+"@x2"),
+            getFloatParam(sXPathExpr+"@y2"));
+     return FRect(pos1, pos2);
 }
 
 xmlNodePtr TrackerConfig::getXmlNode(const std::string& sXPathExpr) const
@@ -214,7 +217,7 @@ xmlNodePtr TrackerConfig::getXmlNode(const std::string& sXPathExpr) const
         throw (Exception(AVG_ERR_OPTION_UNKNOWN, 
                 string("getParam(): cannot find requested element ")+sXPathExpr));
     } else if (nodes->nodeNr > 1) {
-        AVG_TRACE(Logger::WARNING,
+        AVG_LOG_WARNING(
             "getXmlNode(): expression selects more than one node. Returning the first.");
     }
     return nodes->nodeTab[0];
@@ -222,7 +225,7 @@ xmlNodePtr TrackerConfig::getXmlNode(const std::string& sXPathExpr) const
 
 DeDistortPtr TrackerConfig::getTransform() const
 {
-    DPoint CameraExtents = getPointParam("/camera/size/");
+    glm::vec2 CameraExtents = getPointParam("/camera/size/");
     DeDistortPtr pDD(new DeDistort);
     pDD->load(CameraExtents, *this);
     return pDD;
@@ -243,15 +246,15 @@ void TrackerConfig::dump() const
 
 void TrackerConfig::save()
 {
-    AVG_TRACE(Logger::CONFIG, "Saving tracker configuration to " 
-            << m_sFilename << ".");
+    AVG_TRACE(Logger::category::CONFIG, Logger::severity::INFO,
+            "Saving tracker configuration to " << m_sFilename << ".");
 
     if (m_Doc) {
         if (fileExists(m_sFilename)) {
             string sBakFile = m_sFilename + ".bak";
             unlink(sBakFile.c_str());
             if (rename(m_sFilename.c_str(), sBakFile.c_str())) {
-                AVG_TRACE(Logger::WARNING, "Cannot create tracker config backup. Backing "
+                AVG_LOG_WARNING("Cannot create tracker config backup. Backing "
                         "it up on current workdir.");
                 copyFile(m_sFilename, "avgtrackerrc.bak");
             }
