@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,7 @@
 #include "AVGNode.h"
 #include "Player.h"
 
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 #include "KeyEvent.h"
 
 #include "../base/FileHelper.h"
@@ -33,20 +33,17 @@ using namespace std;
 
 namespace avg {
 
-NodeDefinition AVGNode::createDefinition()
+void AVGNode::registerType()
 {
-    return NodeDefinition("avg", Node::buildNode<AVGNode>)
-        .extendDefinition(CanvasNode::createDefinition())
-        .addArg(Arg<string>("onkeyup", ""))
-        .addArg(Arg<string>("onkeydown", ""));
+    TypeDefinition def = TypeDefinition("avg", "canvasbase", 
+            ExportedObject::buildObject<AVGNode>);
+    TypeRegistry::get()->registerType(def);
 }
 
 AVGNode::AVGNode(const ArgList& args)
     : CanvasNode(args)
 {
     args.setMembers(this);
-    addArgEventHandler(Event::KEYUP, Event::NONE, args.getArgVal<string>("onkeyup"));
-    addArgEventHandler(Event::KEYDOWN, Event::NONE, args.getArgVal<string>("onkeydown"));
 }
 
 AVGNode::~AVGNode()
