@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -38,14 +38,14 @@ namespace avg {
 
 typedef std::map<std::string, ArgBasePtr> ArgMap;
 
-class Node;
+class ExportedObject;
 
 class AVG_API ArgList
 {
 public:
     ArgList();
     ArgList(const ArgList& argTemplates, const xmlNodePtr xmlNode);
-    ArgList(const ArgList& argTemplates, const boost::python::dict& PyDict);
+    ArgList(const ArgList& argTemplates, const py::dict& PyDict);
     virtual ~ArgList();
 
     bool hasArg(const std::string& sName) const;
@@ -54,7 +54,7 @@ public:
     template<class T>
     const T& getArgVal(const std::string& sName) const;
     
-    void getOverlayedArgVal(DPoint* pResult, const std::string& sName,
+    void getOverlayedArgVal(glm::vec2* pResult, const std::string& sName,
             const std::string& sOverlay1, const std::string& sOverlay2,
             const std::string& sID) const;
 
@@ -62,12 +62,12 @@ public:
     
     void setArg(const ArgBase& newArg);
     void setArgs(const ArgList& args);
-    void setMembers(Node * pNode) const;
+    void setMembers(ExportedObject * pObj) const;
     
     void copyArgsFrom(const ArgList& argTemplates);
 
 private:
-    void setArgValue(const std::string & sName, const boost::python::object& value);
+    void setArgValue(const std::string & sName, const py::object& value);
     void setArgValue(const std::string & sName, const std::string & sValue);
     ArgMap m_Args;
 };
