@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
 
 #include "CameraNode.h"
 #include "OGLSurface.h"
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 
 #include "../base/Logger.h"
 #include "../base/Exception.h"
@@ -31,6 +31,7 @@
 #include "../graphics/Filterfill.h"
 #include "../graphics/TextureMover.h"
 #include "../graphics/GLTexture.h"
+#include "../graphics/BitmapLoader.h"
 
 #include "../imaging/Camera.h"
 #include "../imaging/FWCamera.h"
@@ -46,15 +47,15 @@ using namespace std;
 
 namespace avg {
 
-NodeDefinition CameraNode::createDefinition()
+void CameraNode::registerType()
 {
-    return NodeDefinition("camera", Node::buildNode<CameraNode>)
-        .extendDefinition(RasterNode::createDefinition())
+    TypeDefinition def = TypeDefinition("camera", "rasternode", 
+            ExportedObject::buildObject<CameraNode>)
         .addArg(Arg<string>("driver", "firewire"))
         .addArg(Arg<string>("device", ""))
         .addArg(Arg<int>("unit", -1))
         .addArg(Arg<bool>("fw800", false))
-        .addArg(Arg<double>("framerate", 15))
+        .addArg(Arg<float>("framerate", 15))
         .addArg(Arg<int>("capturewidth", 640))
         .addArg(Arg<int>("captureheight", 480))
         .addArg(Arg<string>("pixelformat", "RGB"))
@@ -66,6 +67,7 @@ NodeDefinition CameraNode::createDefinition()
         .addArg(Arg<int>("shutter", -1))
         .addArg(Arg<int>("gain", -1))
         .addArg(Arg<int>("strobeduration", -1));
+    TypeRegistry::get()->registerType(def);
 }
 
 CameraNode::CameraNode(const ArgList& args)
@@ -79,7 +81,7 @@ CameraNode::CameraNode(const ArgList& args)
     string sDevice = args.getArgVal<string>("device");
     int unit = args.getArgVal<int>("unit");
     bool bFW800 = args.getArgVal<bool>("fw800");
-    double frameRate = args.getArgVal<double>("framerate");
+    float frameRate = args.getArgVal<float>("framerate");
     int width = args.getArgVal<int>("capturewidth");
     int height = args.getArgVal<int>("captureheight");
     string sPF = args.getArgVal<string>("pixelformat");
@@ -90,7 +92,11 @@ CameraNode::CameraNode(const ArgList& args)
     }
     PixelFormat destPF;
     if (pixelFormatIsColored(camPF)) {
-        destPF = B8G8R8X8;
+        if (BitmapLoader::get()->isBlueFirst()) {
+            destPF = B8G8R8X8;
+        } else {
+            destPF = R8G8B8X8;
+        }
     } else {
         destPF = I8;
     }
@@ -98,8 +104,8 @@ CameraNode::CameraNode(const ArgList& args)
 
     m_pCamera = createCamera(sDriver, sDevice, unit, bFW800, IntPoint(width, height), 
             camPF, destPF, frameRate);
-    AVG_TRACE(Logger::CONFIG, "Got Camera " << m_pCamera->getDevice() << " from driver: "
-            << m_pCamera->getDriverName());
+    AVG_TRACE(Logger::category::CONFIG,Logger::severity::INFO, "Got Camera " <<
+            m_pCamera->getDevice() << " from driver: " << m_pCamera->getDriverName());
     
     m_pCamera->setFeature(CAM_FEATURE_BRIGHTNESS, args.getArgVal<int>("brightness"));
     m_pCamera->setFeature(CAM_FEATURE_EXPOSURE, args.getArgVal<int>("exposure"));
@@ -273,9 +279,10 @@ BitmapPtr CameraNode::getBitmap()
     }
 }
 
-void CameraNode::dumpCameras()
+CamerasInfosVector CameraNode::getCamerasInfos()
 {
-    avg::dumpCameras();
+    CamerasInfosVector camInfos = avg::getCamerasInfos();
+    return camInfos;
 }
 
 void CameraNode::resetFirewireBus()
@@ -283,7 +290,7 @@ void CameraNode::resetFirewireBus()
     FWCamera::resetBus();
 }
 
-double CameraNode::getFPS() const
+float CameraNode::getFPS() const
 {
     return m_pCamera->getFrameRate();
 }
@@ -298,6 +305,7 @@ void CameraNode::open()
     m_pTex = GLTexturePtr(new GLTexture(size, pf, bMipmap));
     m_pTex->enableStreaming();
     getSurface()->create(pf, m_pTex);
+    newSurface();
 
     BitmapPtr pBmp = m_pTex->lockStreamingBmp();
     if (pf == B8G8R8X8 || pf == B8G8R8A8) {
@@ -308,6 +316,7 @@ void CameraNode::open()
         Filter.applyInPlace(pBmp);
     } 
     m_pTex->unlockStreamingBmp(true);
+    setupFX(true);
 }
 
 int CameraNode::getFeature(CameraFeature feature) const
@@ -328,9 +337,10 @@ int CameraNode::getFrameNum() const
 static ProfilingZoneID CameraFetchImage("Camera fetch image");
 static ProfilingZoneID CameraDownloadProfilingZone("Camera tex download");
 
-void CameraNode::preRender()
+void CameraNode::preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+        float parentEffectiveOpacity)
 {
-    Node::preRender();
+    Node::preRender(pVA, bIsParentActive, parentEffectiveOpacity);
     if (isAutoUpdateCameraImage()) {
         ScopeTimer Timer(CameraFetchImage);
         updateToLatestCameraImage();
@@ -346,19 +356,19 @@ void CameraNode::preRender()
         AVG_ASSERT(pBmp->getPixelFormat() == m_pCurBmp->getPixelFormat());
         pBmp->copyPixels(*m_pCurBmp);
         m_pTex->unlockStreamingBmp(true);
-        bind();
         renderFX(getSize(), Pixel32(255, 255, 255, 255), false);
         m_bNewBmp = false;
     }
+    calcVertexArray(pVA);
 }
 
 static ProfilingZoneID CameraProfilingZone("Camera::render");
 
-void CameraNode::render(const DRect& rect)
+void CameraNode::render()
 {
     if (m_bIsPlaying) {
         ScopeTimer Timer(CameraProfilingZone);
-        blt32(getSize(), getEffectiveOpacity(), getBlendMode());
+        blt32(getTransform(), getSize(), getEffectiveOpacity(), getBlendMode());
     }
 }
 
@@ -381,7 +391,7 @@ void CameraNode::updateCameraImage()
 {
     if (!isAutoUpdateCameraImage()) {
         m_pCurBmp = m_pCamera->getImage(false);
-        blt32(getSize(), getEffectiveOpacity(), getBlendMode());
+        blt32(getTransform(), getSize(), getEffectiveOpacity(), getBlendMode());
     }
 }
 
