@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,6 +30,8 @@
 #include "../base/Logger.h"
 #include "../base/ScopeTimer.h"
 
+#include "../graphics/StandardShader.h"
+
 #include <iostream>
 
 using namespace std;
@@ -37,8 +39,6 @@ using namespace boost;
 
 namespace avg {
         
-CanvasPtr Canvas::s_pActiveCanvas;
-
 Canvas::Canvas(Player * pPlayer)
     : m_pPlayer(pPlayer),
       m_bIsPlaying(false),
@@ -57,8 +57,8 @@ void Canvas::setRoot(NodePtr pRootNode)
 {
     assert(!m_pRootNode);
     m_pRootNode = dynamic_pointer_cast<CanvasNode>(pRootNode);
-    m_pRootNode->setParent(DivNodeWeakPtr(), Node::NS_CONNECTED,
-            shared_from_this());
+    CanvasPtr pThis = dynamic_pointer_cast<Canvas>(shared_from_this());
+    m_pRootNode->setParent(0, Node::NS_CONNECTED, pThis);
     registerNode(m_pRootNode);
 }
 
@@ -67,16 +67,20 @@ void Canvas::initPlayback(int multiSampleSamples)
     m_bIsPlaying = true;
     m_pRootNode->connectDisplay();
     m_MultiSampleSamples = multiSampleSamples;
+    m_pVertexArray = VertexArrayPtr(new VertexArray(2000, 3000));
 }
 
-void Canvas::stopPlayback()
+void Canvas::stopPlayback(bool bIsAbort)
 {
     if (m_bIsPlaying) {
-        m_PlaybackEndSignal.emit();
+        if (!bIsAbort) {
+            m_PlaybackEndSignal.emit();
+        }
         m_pRootNode->disconnect(true);
         m_pRootNode = CanvasNodePtr();
         m_IDMap.clear();
         m_bIsPlaying = false;
+        m_pVertexArray = VertexArrayPtr();
     }
 }
 
@@ -85,7 +89,6 @@ NodePtr Canvas::getElementByID(const std::string& id)
     if (m_IDMap.find(id) != m_IDMap.end()) {
         return m_IDMap.find(id)->second;
     } else {
-        AVG_TRACE(Logger::WARNING, "getElementByID(\"" << id << "\") failed.");
         return NodePtr();
     }
 }
@@ -138,25 +141,26 @@ static ProfilingZoneID RenderProfilingZone("Render");
 
 void Canvas::doFrame(bool bPythonAvailable)
 {
-    s_pActiveCanvas = shared_from_this();
     emitPreRenderSignal();
     if (!m_pPlayer->isStopping()) {
         ScopeTimer Timer(RenderProfilingZone);
+        Player::get()->startTraversingTree();
         if (bPythonAvailable) {
             Py_BEGIN_ALLOW_THREADS;
             try {
-                render();
+                renderTree();
             } catch(...) {
                 Py_BLOCK_THREADS;
+                Player::get()->endTraversingTree();
                 throw;
             }
             Py_END_ALLOW_THREADS;
         } else {
-            render();
+            renderTree();
         }
+        Player::get()->endTraversingTree();
     }
     emitFrameEndSignal();
-    s_pActiveCanvas = CanvasPtr();
 }
 
 IntPoint Canvas::getSize() const
@@ -165,20 +169,20 @@ IntPoint Canvas::getSize() const
 }
 static ProfilingZoneID PushClipRectProfilingZone("pushClipRect");
 
-void Canvas::pushClipRect(VertexArrayPtr pVA)
+void Canvas::pushClipRect(const glm::mat4& transform, SubVertexArray& va)
 {
     ScopeTimer timer(PushClipRectProfilingZone);
     m_ClipLevel++;
-    clip(pVA, GL_INCR);
+    clip(transform, va, GL_INCR);
 }
 
 static ProfilingZoneID PopClipRectProfilingZone("popClipRect");
 
-void Canvas::popClipRect(VertexArrayPtr pVA)
+void Canvas::popClipRect(const glm::mat4& transform, SubVertexArray& va)
 {
     ScopeTimer timer(PopClipRectProfilingZone);
     m_ClipLevel--;
-    clip(pVA, GL_DECR);
+    clip(transform, va, GL_DECR);
 }
 
 void Canvas::registerPlaybackEndListener(IPlaybackEndListener* pListener)
@@ -211,102 +215,69 @@ void Canvas::unregisterPreRenderListener(IPreRenderListener* pListener)
     m_PreRenderSignal.disconnect(pListener);
 }
 
-bool Canvas::operator ==(const Canvas& other) const
-{
-    return this == &other;
-}
-
-bool Canvas::operator !=(const Canvas& other) const
-{
-    return this != &other;
-}
-
-long Canvas::getHash() const
-{
-    return long(this);
-}
-
-CanvasPtr Canvas::getActive()
-{
-    return s_pActiveCanvas;
-}
-
 Player* Canvas::getPlayer() const
 {
     return m_pPlayer;
 }
 
-vector<NodeWeakPtr> Canvas::getElementsByPos(const DPoint& pos) const
+vector<NodePtr> Canvas::getElementsByPos(const glm::vec2& pos) const
 {
-    vector<NodeWeakPtr> elements;
+    vector<NodePtr> elements;
     m_pRootNode->getElementsByPos(pos, elements);
     return elements;
 }
 
 static ProfilingZoneID PreRenderProfilingZone("PreRender");
+static ProfilingZoneID VATransferProfilingZone("VA Transfer");
 
-void Canvas::render(IntPoint windowSize, bool bUpsideDown, FBOPtr pFBO,
-        ProfilingZoneID& renderProfilingZone)
+void Canvas::preRender()
 {
+    ScopeTimer Timer(PreRenderProfilingZone);
+    m_pVertexArray->reset();
+    m_pRootNode->preRender(m_pVertexArray, true, 1.0f);
     {
-        ScopeTimer Timer(PreRenderProfilingZone);
-        m_pRootNode->preRender();
-    }
-    if (pFBO) {
-        pFBO->activate();
-    } else {
-        glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, 0);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "Canvas::render: BindFramebuffer()");
-    }
-    if (m_MultiSampleSamples > 1) {
-        glEnable(GL_MULTISAMPLE);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
-                "Canvas::render: glEnable(GL_MULTISAMPLE)");
-    } else {
-        glDisable(GL_MULTISAMPLE);
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL,
-                "Canvas::render: glDisable(GL_MULTISAMPLE)");
-    }
-    clearGLBuffers(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-    glViewport(0, 0, windowSize.x, windowSize.y);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "Canvas::render: glViewport()");
-    glMatrixMode(GL_PROJECTION);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "Canvas::render: glMatrixMode()");
-    glLoadIdentity();
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "Canvas::render: glLoadIdentity()");
-    IntPoint size = IntPoint(m_pRootNode->getSize());
-    if (bUpsideDown) {
-        gluOrtho2D(0, size.x, 0, size.y);
-    } else {
-        gluOrtho2D(0, size.x, size.y, 0);
-    }
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "Canvas::render: gluOrtho2D()");
-    
-    const DRect rc(0,0, size.x, size.y);
-    glMatrixMode(GL_MODELVIEW);
-    {
-        ScopeTimer Timer(renderProfilingZone);
-        m_pRootNode->maybeRender(rc);
-
-        renderOutlines();
+        ScopeTimer Timer(VATransferProfilingZone);
+        m_pVertexArray->update();
     }
 }
 
-void Canvas::renderOutlines()
+void Canvas::render(IntPoint windowSize, bool bOffscreen)
 {
-    GLContext* pContext = GLContext::getCurrent();
+    clearGLBuffers(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT | GL_DEPTH_BUFFER_BIT,
+            !bOffscreen);
+    glViewport(0, 0, windowSize.x, windowSize.y);
+    GLContext::checkError("Canvas::render: glViewport()");
+    glm::vec2 size = m_pRootNode->getSize();
+    glm::mat4 projMat;
+    if (bOffscreen) {
+        projMat = glm::ortho(0.f, size.x, 0.f, size.y);
+    } else {
+        projMat = glm::ortho(0.f, size.x, size.y, 0.f);
+    }
+    m_pVertexArray->activate();
+    m_pRootNode->maybeRender(projMat);
+
+    renderOutlines(projMat);
+}
+
+void Canvas::renderOutlines(const glm::mat4& transform)
+{
+    GLContext* pContext = GLContext::getMain();
     VertexArrayPtr pVA(new VertexArray);
     pContext->setBlendMode(GLContext::BLEND_BLEND, false);
     m_pRootNode->renderOutlines(pVA, Pixel32(0,0,0,0));
-    if (pVA->getCurVert() != 0) {
+    StandardShaderPtr pShader = pContext->getStandardShader();
+    pShader->setTransform(transform);
+    pShader->setUntextured();
+    pShader->setAlpha(0.5f);
+    pShader->activate();
+    if (pVA->getNumVerts() != 0) {
         pVA->update();
-        pContext->enableTexture(false);
-        pContext->enableGLColorArray(true);
         pVA->draw();
     }
 }
 
-void Canvas::clip(VertexArrayPtr pVA, GLenum stencilOp)
+void Canvas::clip(const glm::mat4& transform, SubVertexArray& va, GLenum stencilOp)
 {
     // Disable drawing to color buffer
     glColorMask(0, 0, 0, 0);
@@ -318,7 +289,11 @@ void Canvas::clip(VertexArrayPtr pVA, GLenum stencilOp)
     glStencilFunc(GL_ALWAYS, 0, 0);
     glStencilOp(stencilOp, stencilOp, stencilOp);
 
-    pVA->draw();
+    StandardShaderPtr pShader = GLContext::getMain()->getStandardShader();
+    pShader->setUntextured();
+    pShader->setTransform(transform);
+    pShader->activate();
+    va.draw();
 
     // Set stencil test to only let
     glStencilFunc(GL_LEQUAL, m_ClipLevel, ~0);
