@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,13 +21,16 @@
 
 #ifndef _Canvas_H_
 #define _Canvas_H_
-
+ 
 #include "../api.h"
 
+#include "FontStyle.h"
+
 #include "../base/IPlaybackEndListener.h"
 #include "../base/IFrameEndListener.h"
 #include "../base/IPreRenderListener.h"
 #include "../base/Signal.h"
+#include "../base/GLMHelper.h"
 
 #include "../graphics/OGLHelper.h"
 #include "../graphics/Bitmap.h"
@@ -48,9 +51,9 @@ class ProfilingZoneID;
 class Canvas;
 class FBO;
 class VertexArray;
+class SubVertexArray;
 
 typedef boost::shared_ptr<Node> NodePtr;
-typedef boost::weak_ptr<Node> NodeWeakPtr;
 typedef boost::shared_ptr<CanvasNode> CanvasNodePtr;
 typedef boost::shared_ptr<FBO> FBOPtr;
 typedef boost::shared_ptr<VertexArray> VertexArrayPtr;
@@ -59,14 +62,14 @@ class Canvas;
 typedef boost::shared_ptr<Canvas> CanvasPtr;
 typedef boost::weak_ptr<Canvas> CanvasWeakPtr;
 
-class AVG_API Canvas: public boost::enable_shared_from_this<Canvas>
+class AVG_API Canvas: public ExportedObject
 {
     public:
         Canvas(Player * pPlayer);
         virtual ~Canvas();
         virtual void setRoot(NodePtr pRootNode);
         void initPlayback(int multiSampleSamples);
-        virtual void stopPlayback();
+        virtual void stopPlayback(bool bIsAbort);
        
         CanvasNodePtr getRootNode() const;
         NodePtr getElementByID(const std::string& id);
@@ -76,8 +79,8 @@ class AVG_API Canvas: public boost::enable_shared_from_this<Canvas>
         virtual void doFrame(bool bPythonAvailable);
         IntPoint getSize() const;
         virtual BitmapPtr screenshot() const = 0;
-        virtual void pushClipRect(VertexArrayPtr pVA);
-        virtual void popClipRect(VertexArrayPtr pVA);
+        virtual void pushClipRect(const glm::mat4& transform, SubVertexArray& va);
+        virtual void popClipRect(const glm::mat4& transform, SubVertexArray& va);
 
         void registerPlaybackEndListener(IPlaybackEndListener* pListener);
         void unregisterPlaybackEndListener(IPlaybackEndListener* pListener);
@@ -86,30 +89,25 @@ class AVG_API Canvas: public boost::enable_shared_from_this<Canvas>
         void registerPreRenderListener(IPreRenderListener* pListener);
         void unregisterPreRenderListener(IPreRenderListener* pListener);
 
-        std::vector<NodeWeakPtr> getElementsByPos(const DPoint& Pos) const;
+        std::vector<NodePtr> getElementsByPos(const glm::vec2& Pos) const;
 
-        bool operator ==(const Canvas& other) const;
-        bool operator !=(const Canvas& other) const;
-        long getHash() const;
-
-        static CanvasPtr getActive();
+        virtual void render(IntPoint windowSize, bool bOffscreen);
 
     protected:
         Player * getPlayer() const;
-        void render(IntPoint windowSize, bool bUpsideDown, FBOPtr pFBO,
-                ProfilingZoneID& renderProfilingZone);
+        void preRender();
         void emitPreRenderSignal(); 
         void emitFrameEndSignal();
 
-
     private:
-        virtual void render()=0;
-        void renderOutlines();
+        virtual void renderTree()=0;
+        void renderOutlines(const glm::mat4& transform);
 
-        void clip(VertexArrayPtr pVA, GLenum stencilOp);
+        void clip(const glm::mat4& transform, SubVertexArray& va, GLenum stencilOp);
         Player * m_pPlayer;
         CanvasNodePtr m_pRootNode;
         bool m_bIsPlaying;
+        VertexArrayPtr m_pVertexArray;
        
         typedef std::map<std::string, NodePtr> NodeIDMap;
         NodeIDMap m_IDMap;
@@ -120,8 +118,6 @@ class AVG_API Canvas: public boost::enable_shared_from_this<Canvas>
 
         int m_MultiSampleSamples;
         int m_ClipLevel;
-
-        static CanvasPtr s_pActiveCanvas;
 };
 
 }
