@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,7 @@
 #include "CanvasNode.h"
 #include "Player.h"
 
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 
 #include "../base/FileHelper.h"
 #include "../base/Exception.h"
@@ -31,17 +31,18 @@ using namespace std;
 
 namespace avg {
 
-NodeDefinition CanvasNode::createDefinition()
+void CanvasNode::registerType()
 {
-    return NodeDefinition("canvasbase", Node::buildNode<CanvasNode>)
-        .extendDefinition(DivNode::createDefinition());
+    TypeDefinition def = TypeDefinition("canvasbase", "div", 
+            ExportedObject::buildObject<CanvasNode>);
+    TypeRegistry::get()->registerType(def);
 }
 
 CanvasNode::CanvasNode(const ArgList& args)
     : DivNode(args)
 {
     args.setMembers(this);
-    if (getSize() == DPoint(0, 0)) {
+    if (getSize() == glm::vec2(0, 0)) {
         throw (Exception(AVG_ERR_OUT_OF_RANGE,
                 "<avg> and <canvas> node width and height attributes are mandatory."));
     }
