@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,10 +21,9 @@
 
 #include "DivNode.h"
 #include "Player.h"
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 #include "Canvas.h"
 
-#include "../base/Point.h"
 #include "../base/Exception.h"
 #include "../base/Logger.h"
 #include "../base/StringHelper.h"
@@ -39,36 +38,35 @@
 using namespace std;
 using namespace boost;
 
-#define DEFAULT_SIZE 100000
-
 namespace avg {
 
 
-NodeDefinition DivNode::createDefinition()
+void DivNode::registerType()
 {
     string sChildArray[] = {"image", "div", "canvas", "words", "video", "camera", 
             "panoimage", "sound", "line", "rect", "curve", "polyline", "polygon",
             "circle", "mesh"};
     vector<string> sChildren = vectorFromCArray(
             sizeof(sChildArray) / sizeof(*sChildArray), sChildArray);
-    return NodeDefinition("div", Node::buildNode<DivNode>)
-        .extendDefinition(AreaNode::createDefinition())
+    TypeDefinition def = TypeDefinition("div", "areanode", 
+            ExportedObject::buildObject<DivNode>)
         .addChildren(sChildren)
         .addArg(Arg<bool>("crop", false, false, offsetof(DivNode, m_bCrop)))
-        .addArg(Arg<string>("elementoutlinecolor", "", false, 
-                offsetof(DivNode, m_sElementOutlineColor)))
         .addArg(Arg<UTF8String>("mediadir", "", false, offsetof(DivNode, m_sMediaDir)));
+    TypeRegistry::get()->registerType(def);
 }
 
 DivNode::DivNode(const ArgList& args)
 {
     args.setMembers(this);
-    setElementOutlineColor(m_sElementOutlineColor);
     ObjectCounter::get()->incRef(&typeid(*this));
 }
 
 DivNode::~DivNode()
 {
+    for (unsigned i = 0; i < getNumChildren(); ++i) {
+        getChild(i)->removeParent();
+    }
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
@@ -78,7 +76,6 @@ void DivNode::connectDisplay()
     for (unsigned i = 0; i < getNumChildren(); ++i) {
         getChild(i)->connectDisplay();
     }
-    m_pClipVertexes = VertexArrayPtr(new VertexArray());
 }
 
 void DivNode::connect(CanvasPtr pCanvas)
@@ -94,18 +91,12 @@ void DivNode::disconnect(bool bKill)
     for (unsigned i = 0; i < getNumChildren(); ++i) {
         getChild(i)->disconnect(bKill);
     }
-    if (m_pClipVertexes) {
-        m_pClipVertexes = VertexArrayPtr();
-    }
     AreaNode::disconnect(bKill);
 }
 
-DPoint DivNode::getPivot() const
+glm::vec2 DivNode::getPivot() const
 {
-    DPoint pivot = AreaNode::getPivot();
-    if (pivot == DPoint(DEFAULT_SIZE / 2, DEFAULT_SIZE / 2)) {
-        return DPoint(0, 0);
-    }
+    glm::vec2 pivot = AreaNode::getPivot();
     return pivot;
 }
 
@@ -163,8 +154,7 @@ void DivNode::insertChild(NodePtr pChild, unsigned i)
     if (getState() == NS_CONNECTED || getState() == NS_CANRENDER) {
         getCanvas()->registerNode(pChild);
     }
-    DivNodePtr ptr = dynamic_pointer_cast<DivNode>(shared_from_this());
-    pChild->checkSetParentError(ptr); 
+    pChild->checkSetParentError(this); 
     if (!isChildTypeAllowed(pChild->getTypeStr())) {
         throw(Exception(AVG_ERR_ALREADY_CONNECTED,
                 "Can't insert a node of type "+pChild->getTypeStr()+
@@ -177,7 +167,7 @@ void DivNode::insertChild(NodePtr pChild, unsigned i)
     std::vector<NodePtr>::iterator pos = m_Children.begin()+i;
     m_Children.insert(pos, pChild);
     try {
-        pChild->setParent(ptr, getState(), getCanvas());
+        pChild->setParent(this, getState(), getCanvas());
     } catch (Exception&) {
         m_Children.erase(m_Children.begin()+i);
         throw;
@@ -267,21 +257,6 @@ void DivNode::setCrop(bool bCrop)
     m_bCrop = bCrop;
 }
 
-const std::string& DivNode::getElementOutlineColor() const
-{
-    return m_sElementOutlineColor;
-}
-
-void DivNode::setElementOutlineColor(const std::string& sColor)
-{
-    m_sElementOutlineColor = sColor;
-    if (sColor == "") {
-        m_ElementOutlineColor = Pixel32(0,0,0,0);
-    } else {
-        m_ElementOutlineColor = colorStringToColor(m_sElementOutlineColor);
-    }
-}
-
 const UTF8String& DivNode::getMediaDir() const
 {
     return m_sMediaDir;
@@ -289,88 +264,80 @@ const UTF8String& DivNode::getMediaDir() const
 
 void DivNode::setMediaDir(const UTF8String& sMediaDir)
 {
+//    avgDeprecationWarning("1.7", "DivNode.mediadir", "");
     m_sMediaDir = sMediaDir;
     checkReload();
 }
 
-void DivNode::getElementsByPos(const DPoint& pos, vector<NodeWeakPtr>& pElements)
+void DivNode::getElementsByPos(const glm::vec2& pos, vector<NodePtr>& pElements)
 {
     if (reactsToMouseEvents() &&
-            ((getSize() == DPoint(DEFAULT_SIZE, DEFAULT_SIZE) ||
+            ((getSize() == glm::vec2(0,0) ||
              (pos.x >= 0 && pos.y >= 0 && pos.x < getSize().x && pos.y < getSize().y))))
     {
         for (int i = getNumChildren()-1; i >= 0; i--) {
             NodePtr pCurChild = getChild(i);
-            DPoint relPos = pCurChild->toLocal(pos);
+            glm::vec2 relPos = pCurChild->toLocal(pos);
             pCurChild->getElementsByPos(relPos, pElements);
             if (!pElements.empty()) {
-                pElements.push_back(shared_from_this());
+                pElements.push_back(getSharedThis());
                 return;
             }
         }
         // pos isn't in any of the children.
-        if (getSize() != DPoint(DEFAULT_SIZE, DEFAULT_SIZE)) {
+        if (getSize() != glm::vec2(0,0)) {
             // Explicit width/height given for div - div reacts on its own.
-            pElements.push_back(shared_from_this());
+            pElements.push_back(getSharedThis());
         }
     }
 }
 
-void DivNode::preRender()
+void DivNode::preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+        float parentEffectiveOpacity)
 {
-    Node::preRender();
+    Node::preRender(pVA, bIsParentActive, parentEffectiveOpacity);
+    if (getCrop() && getSize() != glm::vec2(0,0)) {
+        pVA->startSubVA(m_ClipVA);
+        glm::vec2 viewport = getSize();
+        m_ClipVA.appendPos(glm::vec2(0,0), glm::vec2(0,0), Pixel32(0,0,0,0));
+        m_ClipVA.appendPos(glm::vec2(0,viewport.y), glm::vec2(0,0), Pixel32(0,0,0,0));
+        m_ClipVA.appendPos(glm::vec2(viewport.x,0), glm::vec2(0,0), Pixel32(0,0,0,0));
+        m_ClipVA.appendPos(viewport, glm::vec2(0,0), Pixel32(0,0,0,0));
+        m_ClipVA.appendQuadIndexes(0, 1, 2, 3);
+    }
     for (unsigned i = 0; i < getNumChildren(); i++) {
-        getChild(i)->preRender();
+        getChild(i)->preRender(pVA, bIsParentActive, getEffectiveOpacity());
     }
 }
 
-void DivNode::render(const DRect& rect)
+void DivNode::render()
 {
-    DPoint viewport = getSize();
-    
-    m_pClipVertexes->reset();
-    m_pClipVertexes->appendPos(DPoint(0,0), DPoint(0,0), Pixel32(0,0,0,0));
-    m_pClipVertexes->appendPos(DPoint(0,viewport.y), DPoint(0,0), Pixel32(0,0,0,0));
-    m_pClipVertexes->appendPos(DPoint(viewport.x,0), DPoint(0,0), Pixel32(0,0,0,0));
-    m_pClipVertexes->appendPos(viewport, DPoint(0,0), Pixel32(0,0,0,0));
-    m_pClipVertexes->appendQuadIndexes(0, 1, 2, 3);
-
-    if (getCrop()) {
-        getCanvas()->pushClipRect(m_pClipVertexes);
+    const glm::mat4& transform = getTransform();
+    if (getCrop() && getSize() != glm::vec2(0,0)) {
+        getCanvas()->pushClipRect(transform, m_ClipVA);
     }
     for (unsigned i = 0; i < getNumChildren(); i++) {
-        getChild(i)->maybeRender(rect);
+        getChild(i)->maybeRender(transform);
     }
-    if (getCrop()) {
-        getCanvas()->popClipRect(m_pClipVertexes);
+    if (getCrop() && getSize() != glm::vec2(0,0)) {
+        getCanvas()->popClipRect(transform, m_ClipVA);
     }
 }
 
-void DivNode::renderOutlines(const VertexArrayPtr& pVA, Pixel32 color)
+void DivNode::renderOutlines(const VertexArrayPtr& pVA, Pixel32 parentColor)
 {
-    Pixel32 effColor = color;
-    if (m_ElementOutlineColor != Pixel32(0,0,0,0)) {
-        effColor = m_ElementOutlineColor;
-        effColor.setA(128);
-    }
+    Pixel32 effColor = getEffectiveOutlineColor(parentColor);
     if (effColor != Pixel32(0,0,0,0)) {
-        DPoint size = getSize();
-        if (size == DPoint(DEFAULT_SIZE, DEFAULT_SIZE)) {
-            DPoint p0 = getAbsPos(DPoint(-4, 0.5));
-            DPoint p1 = getAbsPos(DPoint(5, 0.5));
-            DPoint p2 = getAbsPos(DPoint(0.5, -4));
-            DPoint p3 = getAbsPos(DPoint(0.5, 5));
+        glm::vec2 size = getSize();
+        if (size == glm::vec2(0,0)) {
+            glm::vec2 p0 = getAbsPos(glm::vec2(-4, 0.5));
+            glm::vec2 p1 = getAbsPos(glm::vec2(5, 0.5));
+            glm::vec2 p2 = getAbsPos(glm::vec2(0.5, -4));
+            glm::vec2 p3 = getAbsPos(glm::vec2(0.5, 5));
             pVA->addLineData(effColor, p0, p1, 1);
             pVA->addLineData(effColor, p2, p3, 1);
         } else {
-            DPoint p0 = getAbsPos(DPoint(0.5, 0.5));
-            DPoint p1 = getAbsPos(DPoint(size.x+0.5,0.5));
-            DPoint p2 = getAbsPos(DPoint(size.x+0.5,size.y+0.5));
-            DPoint p3 = getAbsPos(DPoint(0.5,size.y+0.5));
-            pVA->addLineData(effColor, p0, p1, 1);
-            pVA->addLineData(effColor, p1, p2, 1);
-            pVA->addLineData(effColor, p2, p3, 1);
-            pVA->addLineData(effColor, p3, p0, 1);
+            AreaNode::renderOutlines(pVA, parentColor);
         }
     }
     for (unsigned i = 0; i < getNumChildren(); i++) {
@@ -414,7 +381,7 @@ string DivNode::dump(int indent)
 
 IntPoint DivNode::getMediaSize()
 {
-    return IntPoint(DEFAULT_SIZE, DEFAULT_SIZE);
+    return IntPoint(0, 0);
 }
  
 bool DivNode::isChildTypeAllowed(const string& sType)
