@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -38,17 +38,19 @@ class Player;
 
 class AVG_API EventDispatcher {
     public:
-        EventDispatcher(Player* pPlayer);
+        EventDispatcher(Player* pPlayer, bool bMouseEnabled);
         virtual ~EventDispatcher();
         void dispatch();
         
         void addInputDevice(IInputDevicePtr pInputDevice);
 
         void sendEvent(EventPtr pEvent);
+        void enableMouse(bool bEnabled);
         ContactPtr getContact(int id);
 
     private:
         void handleEvent(EventPtr pEvent);
+        bool processEventHook(EventPtr pEvent);
         void testAddContact(EventPtr pEvent);
         void testRemoveContact(EventPtr pEvent);
 
@@ -56,6 +58,7 @@ class AVG_API EventDispatcher {
         Player* m_pPlayer;
         std::map<int, ContactPtr> m_ContactMap;
         int m_NumMouseButtonsDown;
+        bool m_bMouseEnabled;
 };
 typedef boost::shared_ptr<EventDispatcher> EventDispatcherPtr;
 
