@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,8 +29,9 @@ namespace avg {
 
 using namespace std;
 
-FXNode::FXNode() 
+FXNode::FXNode(bool bSupportsGLES) 
     : m_Size(0, 0),
+      m_bSupportsGLES(bSupportsGLES),
       m_bDirty(true)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
@@ -43,6 +44,7 @@ FXNode::~FXNode()
 
 void FXNode::connect()
 {
+    checkGLES();
     if (m_Size != IntPoint(0,0)) {
         m_pFilter = createFilter(m_Size);
     }
@@ -66,7 +68,7 @@ void FXNode::setSize(const IntPoint& newSize)
 void FXNode::apply(GLTexturePtr pSrcTex)
 {
     // blt overwrites everything, so no glClear necessary before.
-    GLContext::getCurrent()->setBlendMode(GLContext::BLEND_COPY);
+    GLContext::getMain()->setBlendMode(GLContext::BLEND_COPY);
     m_pFilter->apply(pSrcTex);
 }
 
@@ -80,7 +82,7 @@ BitmapPtr FXNode::getImage()
     return m_pFilter->getImage();
 }
 
-DRect FXNode::getRelDestRect() const
+FRect FXNode::getRelDestRect() const
 {
     return m_pFilter->getRelDestRect();
 }
@@ -105,4 +107,11 @@ void FXNode::setDirty()
     m_bDirty = true;
 }
 
+void FXNode::checkGLES() const
+{
+    if (!m_bSupportsGLES && GLContext::getCurrent()->isGLES()) {
+        throw Exception(AVG_ERR_UNSUPPORTED, "This effect is unsupported under OpenGL ES.");
+    }
+}
+
 }
