@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,10 +25,12 @@
 #include "Player.h"
 
 #include "../base/Exception.h"
-#include "../base/ProfilingZone.h"
+#include "../base/ProfilingZoneID.h"
 #include "../base/ObjectCounter.h"
+#include "../base/ScopeTimer.h"
 
 #include "../graphics/FilterUnmultiplyAlpha.h"
+#include "../graphics/BitmapLoader.h"
 
 #include <iostream>
 
@@ -62,38 +64,40 @@ void OffscreenCanvas::setRoot(NodePtr pRootNode)
 void OffscreenCanvas::initPlayback()
 {
     m_bUseMipmaps = getMipmap();
-    m_pFBO = FBOPtr(new FBO(getSize(), B8G8R8A8, 1, getMultiSampleSamples(), true,
-            m_bUseMipmaps));
+    PixelFormat pf;
+    if (BitmapLoader::get()->isBlueFirst()) {
+        pf = B8G8R8A8;
+    } else {
+        pf = R8G8B8A8;
+    }
+    bool bUseDepthBuffer = GLContext::getMain()->useDepthBuffer();
+    m_pFBO = FBOPtr(new FBO(getSize(), pf, 1, getMultiSampleSamples(), bUseDepthBuffer,
+            true, m_bUseMipmaps));
     Canvas::initPlayback(getMultiSampleSamples());
     m_bIsRendered = false;
 }
 
-void OffscreenCanvas::stopPlayback()
+void OffscreenCanvas::stopPlayback(bool bIsAbort)
 {
     m_pFBO = FBOPtr();
-    Canvas::stopPlayback();
+    Canvas::stopPlayback(bIsAbort);
     m_bIsRendered = false;
 }
 
 BitmapPtr OffscreenCanvas::screenshot() const
 {
-    return screenshot(false);
+    BitmapPtr pBmp = screenshotIgnoreAlpha();
+    FilterUnmultiplyAlpha().applyInPlace(pBmp);
+    return pBmp;
 }
 
-static ProfilingZoneID OffscreenRenderProfilingZone("Render OffscreenCanvas");
-
-BitmapPtr OffscreenCanvas::screenshot(bool bIgnoreAlpha) const
+BitmapPtr OffscreenCanvas::screenshotIgnoreAlpha() const
 {
     if (!isRunning() || !m_bIsRendered) {
         throw(Exception(AVG_ERR_UNSUPPORTED,
                 "OffscreenCanvas::screenshot(): Canvas has not been rendered. No screenshot available"));
     }
     BitmapPtr pBmp = m_pFBO->getImage(0);
-    if (bIgnoreAlpha) {
-        pBmp->setPixelFormat(B8G8R8X8);
-    } else {
-        FilterUnmultiplyAlpha().applyInPlace(pBmp);
-    }
     return pBmp;
 }
 
@@ -126,7 +130,7 @@ void OffscreenCanvas::setAutoRender(bool bAutoRender)
 void OffscreenCanvas::manualRender()
 {
     emitPreRenderSignal(); 
-    render(); 
+    renderTree(); 
     emitFrameEndSignal(); 
 }
 
@@ -184,10 +188,13 @@ bool OffscreenCanvas::isCameraImageAvailable() const
 
 void OffscreenCanvas::addDependentCanvas(CanvasPtr pCanvas)
 {
-    AVG_ASSERT(!(pCanvas == shared_from_this()));
     m_pDependentCanvases.push_back(pCanvas);
-    Player::get()->newCanvasDependency(
-            dynamic_pointer_cast<OffscreenCanvas>(shared_from_this()));
+    try {
+        Player::get()->newCanvasDependency();
+    } catch (Exception&) {
+        m_pDependentCanvases.pop_back();
+        throw;
+    }
 }
 
 void OffscreenCanvas::removeDependentCanvas(CanvasPtr pCanvas)
@@ -202,14 +209,9 @@ void OffscreenCanvas::removeDependentCanvas(CanvasPtr pCanvas)
     AVG_ASSERT(false);
 }
 
-bool OffscreenCanvas::hasDependentCanvas(CanvasPtr pCanvas) const
+const vector<CanvasPtr>& OffscreenCanvas::getDependentCanvases() const
 {
-    for (unsigned i = 0; i < m_pDependentCanvases.size(); ++i) {
-        if (pCanvas == m_pDependentCanvases[i]) {
-            return true;
-        }
-    }
-    return false;
+    return m_pDependentCanvases;
 }
 
 unsigned OffscreenCanvas::getNumDependentCanvases() const
@@ -223,8 +225,11 @@ bool OffscreenCanvas::isSupported()
         throw(Exception(AVG_ERR_UNSUPPORTED, 
                 "OffscreenCanvas::isSupported(): Player.play() needs to be called before support can be queried."));
     }
-
-    return FBO::isFBOSupported() && FBO::isPackedDepthStencilSupported();
+    if (GLContext::getMain()->isGLES()) {
+        return true;
+    } else {
+        return FBO::isFBOSupported() && FBO::isPackedDepthStencilSupported();
+    }
 }
 
 bool OffscreenCanvas::isMultisampleSupported()
@@ -245,14 +250,20 @@ void OffscreenCanvas::dump() const
     }
 }
 
-void OffscreenCanvas::render()
+static ProfilingZoneID OffscreenRenderProfilingZone("Render OffscreenCanvas");
+
+void OffscreenCanvas::renderTree()
 {
     if (!isRunning()) {
         throw(Exception(AVG_ERR_UNSUPPORTED, 
-                "OffscreenCanvas::render(): Player.play() needs to be called before rendering offscreen canvases."));
+                "OffscreenCanvas::renderTree(): Player.play() needs to be called before rendering offscreen canvases."));
+    }
+    preRender();
+    m_pFBO->activate();
+    {
+        ScopeTimer Timer(OffscreenRenderProfilingZone);
+        Canvas::render(IntPoint(getRootNode()->getSize()), true);
     }
-    Canvas::render(IntPoint(getRootNode()->getSize()), true, m_pFBO, 
-            OffscreenRenderProfilingZone);
     m_pFBO->copyToDestTexture();
     m_bIsRendered = true;
 }
