@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,7 +32,7 @@ namespace avg {
 class AVG_API OffscreenCanvasNode : public CanvasNode
 {
     public:
-        static NodeDefinition createDefinition();
+        static void registerType();
         
         OffscreenCanvasNode(const ArgList& args);
         virtual ~OffscreenCanvasNode();
