@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,8 +23,9 @@
 #define _Player_H_
 
 #include "../api.h"
+#include "Publisher.h"
 #include "Timeout.h"
-#include "NodeRegistry.h"
+#include "TypeRegistry.h"
 #include "DisplayParams.h"
 #include "CursorState.h"
 #include "TestHelper.h"
@@ -35,6 +36,7 @@
 
 #include <libxml/parser.h>
 #include <boost/shared_ptr.hpp>
+#include <boost/thread/thread.hpp>
 
 #include <string>
 #include <vector>
@@ -57,6 +59,7 @@ class EventDispatcher;
 class MouseEvent;
 class CursorEvent;
 class SDLDisplayEngine;
+class Display;
 
 typedef boost::shared_ptr<Node> NodePtr;
 typedef boost::weak_ptr<Node> NodeWeakPtr;
@@ -68,8 +71,9 @@ typedef boost::shared_ptr<EventDispatcher> EventDispatcherPtr;
 typedef boost::shared_ptr<MouseEvent> MouseEventPtr;
 typedef boost::shared_ptr<CursorEvent> CursorEventPtr;
 typedef boost::shared_ptr<SDLDisplayEngine> SDLDisplayEnginePtr;
+typedef boost::shared_ptr<Display> DisplayPtr;
 
-class AVG_API Player
+class AVG_API Player: public Publisher
 {
     public:
         Player();
@@ -82,60 +86,66 @@ class AVG_API Player
         bool isFullscreen();
         void setWindowFrame(bool bHasWindowFrame);
         void setWindowPos(int x=0, int y=0);
-        void setOGLOptions(bool bUsePOTTextures, bool bUseShaders, 
-                bool bUsePixelBuffers, int multiSampleSamples);
+        void setWindowTitle(const std::string& sTitle);
+        void useGLES(bool bGLES);
+        void setOGLOptions(bool bUsePOTTextures, bool bUsePixelBuffers, 
+                int multiSampleSamples, GLConfig::ShaderUsage shaderUsage,
+                bool bUseDebugContext);
         void setMultiSampleSamples(int multiSampleSamples);
         void setAudioOptions(int samplerate, int channels);
-        DPoint getScreenResolution();
-        double getPixelsPerMM();
-        DPoint getPhysicalScreenDimensions();
-        void assumePixelsPerMM(double ppmm);
+        void enableGLErrorChecks(bool bEnable);
+        glm::vec2 getScreenResolution();
+        float getPixelsPerMM();
+        glm::vec2 getPhysicalScreenDimensions();
+        void assumePixelsPerMM(float ppmm);
 
         CanvasPtr loadFile(const std::string& sFilename);
         CanvasPtr loadString(const std::string& sAVG);
 
         OffscreenCanvasPtr loadCanvasFile(const std::string& sFilename);
         OffscreenCanvasPtr loadCanvasString(const std::string& sAVG);
-        CanvasPtr createMainCanvas(const boost::python::dict& params);
-        OffscreenCanvasPtr createCanvas(const boost::python::dict& params);
+        CanvasPtr createMainCanvas(const py::dict& params);
+        OffscreenCanvasPtr createCanvas(const py::dict& params);
         void deleteCanvas(const std::string& sID);
         CanvasPtr getMainCanvas() const;
         OffscreenCanvasPtr getCanvas(const std::string& sID) const;
-        void newCanvasDependency(const OffscreenCanvasPtr pCanvas);
+        void newCanvasDependency();
 
         void play();
         void stop();
         bool isStopping();
-        void initPlayback();
-        void cleanup();
+        void initPlayback(const std::string& sShaderPath = "");
+        void cleanup(bool bIsAbort);
         bool isPlaying();
-        void setFramerate(double rate);
+        void setFramerate(float rate);
         void setVBlankFramerate(int rate);
-        double getEffectiveFramerate();
+        float getEffectiveFramerate();
         TestHelper * getTestHelper();
-        void setFakeFPS(double fps);
+        void setFakeFPS(float fps);
         long long getFrameTime();
-        double getFrameDuration();
+        float getFrameDuration();
 
-        void registerNodeType(NodeDefinition Def, const char* pParentNames[] = 0);
-        
-        NodePtr createNode(const std::string& sType, const boost::python::dict& PyDict);
+        NodePtr createNode(const std::string& sType, const py::dict& PyDict,
+                const py::object& self=py::object());
         NodePtr createNodeFromXmlString(const std::string& sXML);
         
         int setInterval(int time, PyObject * pyfunc);
         int setTimeout(int time, PyObject * pyfunc);
         int setOnFrameHandler(PyObject * pyfunc);
         bool clearInterval(int id);
+        void callFromThread(PyObject * pyfunc);
 
         void addInputDevice(IInputDevicePtr pSource);
         MouseEventPtr getMouseState() const;
-        TrackerInputDevice * addTracker();
+        EventPtr getCurrentEvent() const;
         TrackerInputDevice * getTracker();
         void enableMultitouch();
+        void enableMouse(bool enabled);
         bool isMultitouchAvailable() const;
         void setEventCapture(NodePtr pNode, int cursorID);
         void releaseEventCapture(int cursorID);
         bool isCaptured(int cursorID);
+        void removeDeadEventCaptures();
         EventPtr getCurEvent() const;
         void setMousePos(const IntPoint& pos);
         int getKeyModifierState() const;
@@ -143,29 +153,38 @@ class AVG_API Player
         BitmapPtr screenshot();
         void setCursor(const Bitmap* pBmp, IntPoint hotSpot);
         void showCursor(bool bShow);
+        bool isCursorShown();
 
         NodePtr getElementByID(const std::string& id);
         AVGNodePtr getRootNode();
         void doFrame(bool bFirstFrame);
-        double getFramerate();
-        double getVideoRefreshRate();
-        bool isUsingShaders();
-        void setGamma(double red, double green, double blue);
+        float getFramerate();
+        float getVideoRefreshRate();
+        size_t getVideoMemInstalled();
+        size_t getVideoMemUsed();
+        void setGamma(float red, float green, float blue);
         SDLDisplayEngine * getDisplayEngine() const;
+        void keepWindowOpen();
         void setStopOnEscape(bool bStop);
         bool getStopOnEscape() const;
-        void setVolume(double volume);
-        double getVolume() const;
+        void setVolume(float volume);
+        float getVolume() const;
+        std::string getConfigOption(const std::string& sSubsys, const std::string& sName)
+                const;
+        bool isUsingGLES() const;
+        bool areFullShadersSupported() const;
 
         OffscreenCanvasPtr getCanvasFromURL(const std::string& sURL);
 
         std::string getCurDirName();
         std::string getRootMediaDir();
-        const NodeDefinition& getNodeDef(const std::string& sType);
 
         void disablePython();
+        void startTraversingTree();
+        void endTraversingTree();
+        bool isTraversingTree() const;
 
-        boost::python::object loadPlugin(const std::string& name);
+        py::object loadPlugin(const std::string& name);
         void setPluginPath(const std::string& newPath);
         std::string getPluginPath() const;
         
@@ -183,14 +202,13 @@ class AVG_API Player
 
     private:
         void initConfig();
-        void initGraphics();
+        void initGraphics(const std::string& sShaderPath);
         void initAudio();
-
-        void updateDTD();
+        void initMainCanvas(NodePtr pRootNode);
 
         NodePtr loadMainNodeFromFile(const std::string& sFilename);
         NodePtr loadMainNodeFromString(const std::string& sAVG);
-        NodePtr internalLoad(const std::string& sAVG);
+        NodePtr internalLoad(const std::string& sAVG, const std::string& sFilename);
         SDLDisplayEnginePtr safeGetDisplayEngine();
 
         NodePtr createNodeFromXml(const xmlDocPtr xmlDoc,
@@ -210,42 +228,44 @@ class AVG_API Player
         MainCanvasPtr m_pMainCanvas;
 
         SDLDisplayEnginePtr m_pDisplayEngine;
+        bool m_bDisplayEngineBroken;
         TestHelperPtr m_pTestHelper;
        
         std::string m_CurDirName;
+        bool m_bIsTraversingTree;
         bool m_bStopping;
-        NodeRegistry m_NodeRegistry;
 
         IInputDevicePtr m_pMultitouchInputDevice;
 
+        // Timeout handling
+        int internalSetTimeout(int time, PyObject * pyfunc, bool bIsInterval);
         int addTimeout(Timeout* pTimeout);
-        void removeTimeout(Timeout* pTimeout);
         void handleTimers();
         bool m_bInHandleTimers;
         bool m_bCurrentTimeoutDeleted;
 
         std::vector<Timeout *> m_PendingTimeouts;
         std::vector<Timeout *> m_NewTimeouts; // Timeouts to be added this frame.
+        std::vector<Timeout *> m_AsyncCalls;
+        boost::mutex m_AsyncCallMutex;
 
         // Configuration variables.
         DisplayParams m_DP;
         AudioParams m_AP;
         GLConfig m_GLConfig;
 
+        bool m_bKeepWindowOpen;
         bool m_bStopOnEscape;
         bool m_bIsPlaying;
 
         // Time calculation
         bool m_bFakeFPS;
-        double m_FakeFPS;
+        float m_FakeFPS;
         long long m_FrameTime;
         long long m_PlayStartTime;
         long long m_NumFrames;
 
-        double m_Volume;
-
-        bool m_bDirtyDTD;
-        xmlDtdPtr m_dtd;
+        float m_Volume;
 
         bool m_bPythonAvailable;
 
@@ -258,7 +278,7 @@ class AVG_API Player
         struct EventCaptureInfo {
             EventCaptureInfo(const NodeWeakPtr& pNode);
 
-            NodeWeakPtr m_pNode;
+            NodePtr m_pNode;
             int m_CaptureCount;
         };
         typedef boost::shared_ptr<EventCaptureInfo> EventCaptureInfoPtr;
@@ -266,11 +286,13 @@ class AVG_API Player
         std::map<int, EventCaptureInfoPtr> m_EventCaptureInfoMap;
 
         MouseEventPtr m_pLastMouseEvent;
+        EventPtr m_pCurrentEvent;
 
         // The indexes of this map are cursorids.
         std::map<int, CursorStatePtr> m_pLastCursorStates;
 
         PyObject * m_EventHookPyFunc;
+        bool m_bMouseEnabled;
 };
 
 }
