@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 
 #include "FXNode.h"
 #include "../graphics/GPUShadowFilter.h"
+#include "../base/GLMHelper.h"
 
 #include <boost/shared_ptr.hpp>
 #include <string>
@@ -34,20 +35,19 @@ namespace avg {
 
 class AVG_API ShadowFXNode: public FXNode {
 public:
-    ShadowFXNode();
+    ShadowFXNode(glm::vec2 offset=glm::vec2(0,0), float radius=1.f, float opacity=1.f,
+            std::string sColor="FFFFFF");
     virtual ~ShadowFXNode();
 
     virtual void connect();
     virtual void disconnect();
-    void setParams(const DPoint& offset, double stdDev, double opacity, 
-            const std::string& sColor);
 
-    void setOffset(const DPoint& offset);
-    DPoint getOffset() const;
-    void setRadius(double radius);
-    double getRadius() const;
-    void setOpacity(double opacity);
-    double getOpacity() const;
+    void setOffset(const glm::vec2& offset);
+    glm::vec2 getOffset() const;
+    void setRadius(float radius);
+    float getRadius() const;
+    void setOpacity(float opacity);
+    float getOpacity() const;
     void setColor(const std::string& sColor);
     std::string getColor() const;
 
@@ -57,9 +57,9 @@ private:
 
     GPUShadowFilterPtr m_pFilter;
 
-    DPoint m_Offset;
-    double m_StdDev;
-    double m_Opacity;
+    glm::vec2 m_Offset;
+    float m_StdDev;
+    float m_Opacity;
     std::string m_sColorName;
     Pixel32 m_Color;
 };
