@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,11 +25,17 @@
 #include "../base/Exception.h"
 #include "../base/Logger.h"
 
+#include "../graphics/GLConfig.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string>
 
+#ifdef WIN32
+#include <direct.h>
+#endif
+
 using namespace avg;
 using namespace std;
 
@@ -49,11 +55,18 @@ public:
         "      <words text=\"foo\"/>"
         "    </avg>"
         );
+        player.setOGLOptions(false, true, 1, GLConfig::AUTO, true);
+        GLContext::enableErrorChecks(true);
         player.disablePython();
         if (!getenv("AVG_CONSOLE_TEST")) {
-            player.initPlayback();
+#ifdef WIN32
+            char sz[1024];
+            _getcwd(sz, 1024);
+            cerr << "Current directory: " << sz << endl;
+#endif
+            player.initPlayback("../graphics/shaders/");
             player.doFrame(false);
-            player.cleanup();
+            player.cleanup(false);
         }
         try {
             throw bad_cast();
