@@ -1,5 +1,5 @@
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -21,14 +21,12 @@
 #
 
 import sys, os
-from libavg import avg, AVGApp, AVGAppStarter
+from libavg import avg, AVGApp, player
 
 import coordcalibrator
 import apphelpers
 
 mediadir = os.path.join(os.path.dirname(__file__), 'data')
-g_Player = avg.Player.get()
-g_Log = avg.Logger.get()
 g_KbManager = apphelpers.KeyboardManager.get()
 
 
@@ -79,7 +77,7 @@ def camera_setup(CameraType):
              'min':16, 'max':683, 'increment':2, 'precision':0},
         ]
     else:
-        g_Log.trace(g_Log.ERROR, "Unknown CameraType %s" % CameraType)
+        avg.logger.error("Unknown CameraType %s" % CameraType)
         sys.exit()
 
     paramList.extend([
@@ -165,7 +163,7 @@ class Calibrator(AVGApp):
         self.paramList = camera_setup(CameraType)
         self.parentNode=parentNode
         self.appStarter = appStarter
-        self.mainNode = g_Player.createNode(
+        self.mainNode = player.createNode(
         """
         <div active="False" opacity="0">
             <image width="1280" height="800" href="black.png"/>
@@ -173,69 +171,66 @@ class Calibrator(AVGApp):
                     sensitive="false" opacity="1"/>
             <words id="cal_fps" x="30" y="30" color="00FF00" text=""/>
             <words id="cal_notification" x="390" y="390" width="500" fontsize="18"
-                    font="Zurich Ex BT" color="ff3333" alignment="center" />
+                    color="ff3333" alignment="center" />
             <div id="cal_gui" x="30" y="540">
                 <image id="cal_shadow" x="0" y="13" width="500" height="150" 
                         href="black.png" opacity="0.6"/>
                 
-                <words x="2" y="13" text="camera" fontsize="16" font="Zurich Ex BT" 
-                        color="00FF00"/>
+                <words x="2" y="13" text="camera" fontsize="16" color="00FF00"/>
                 <image x="2" y="32" href="CamImgBorder.png"/>
                 <image id="cal_camera" x="4" y="34" width="160" height="120"/>
 
-                <words x="168" y="13" text="nohistory" fontsize="16" font="Zurich Ex BT" 
-                        color="00FF00"/>
+                <words x="168" y="13" text="nohistory" fontsize="16" color="00FF00"/>
                 <image x="168" y="32" href="CamImgBorder.png"/>
                 <image id="cal_nohistory" x="170" y="34" width="160" height="120"/>
                 
-                <words x="334" y="13" text="histogram" fontsize="16" font="Zurich Ex BT" 
-                        color="00FF00"/>
+                <words x="334" y="13" text="histogram" fontsize="16" color="00FF00"/>
                 <image x="334" y="32" href="CamImgBorder.png"/>
                 <image id="cal_histogram" x="336" y="34" width="160" height="120"/>
 
                 <div id="cal_params" y="170" opacity="0.9">
                 <image id="cal_shadow2" width="750" height="65" href="black.png" opacity="0.6"/>
                     <div id="cal_paramdiv0" x="2">
-                        <words text="camera" y="0" fontsize="10" font="Zurich Ex BT" color="00ff00" /> 
-                        <words id="cal_param0" y="12" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param1" y="24" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param2" y="36" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param3" y="48" fontsize="10" font="Zurich Ex BT" /> 
+                        <words text="camera" y="0" fontsize="10" color="00ff00" /> 
+                        <words id="cal_param0" y="12" fontsize="10"/> 
+                        <words id="cal_param1" y="24" fontsize="10"/> 
+                        <words id="cal_param2" y="36" fontsize="10"/> 
+                        <words id="cal_param3" y="48" fontsize="10"/> 
                     </div>
                     <div id="cal_paramdiv1" x="80">
-                        <words text="touch" y="0" fontsize="10" font="Zurich Ex BT" color="00ff00" /> 
-                        <words id="cal_param4" y="12" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param5" y="24" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param6" y="36" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param7" y="48" fontsize="10" font="Zurich Ex BT" /> 
+                        <words text="touch" y="0" fontsize="10" color="00ff00" /> 
+                        <words id="cal_param4" y="12" fontsize="10"/> 
+                        <words id="cal_param5" y="24" fontsize="10"/> 
+                        <words id="cal_param6" y="36" fontsize="10"/> 
+                        <words id="cal_param7" y="48" fontsize="10"/> 
                     </div>
                     <div id="cal_paramdiv2" x="200">
-                        <words id="cal_param8" y="0" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param9" y="12" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param10" y="24" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param11" y="36" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param12" y="48" fontsize="10" font="Zurich Ex BT" /> 
+                        <words id="cal_param8" y="0" fontsize="10"/> 
+                        <words id="cal_param9" y="12" fontsize="10"/> 
+                        <words id="cal_param10" y="24" fontsize="10"/> 
+                        <words id="cal_param11" y="36" fontsize="10"/> 
+                        <words id="cal_param12" y="48" fontsize="10"/> 
                     </div>
                     <div id="cal_paramdiv3" x="350">
-                        <words text="track" y="0" fontsize="10" font="Zurich Ex BT" color="00ff00" /> 
-                        <words id="cal_param13" y="12" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param14" y="24" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param15" y="36" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param16" y="48" fontsize="10" font="Zurich Ex BT" /> 
+                        <words text="track" y="0" fontsize="10" color="00ff00" /> 
+                        <words id="cal_param13" y="12" fontsize="10"/> 
+                        <words id="cal_param14" y="24" fontsize="10"/> 
+                        <words id="cal_param15" y="36" fontsize="10"/> 
+                        <words id="cal_param16" y="48" fontsize="10"/> 
                     </div>
                     <div id="cal_paramdiv4" x="500">
-                        <words id="cal_param17" y="0" fontsize="10" font="Zurich Ex BT" /> 
-                        <words text="distort" y="12" fontsize="10" font="Zurich Ex BT" color="00ff00" /> 
-                        <words id="cal_param18" y="24" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param19" y="36" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param20" y="48" fontsize="10" font="Zurich Ex BT" /> 
+                        <words id="cal_param17" y="0" fontsize="10"/> 
+                        <words text="distort" y="12" fontsize="10" color="00ff00"/> 
+                        <words id="cal_param18" y="24" fontsize="10"/> 
+                        <words id="cal_param19" y="36" fontsize="10"/> 
+                        <words id="cal_param20" y="48" fontsize="10"/> 
                     </div>
                     <div id="cal_paramdiv5" x="650">
-                        <words id="cal_param21" y="0" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param22" y="12" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param23" y="24" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param24" y="36" fontsize="10" font="Zurich Ex BT" /> 
-                        <words id="cal_param25" y="48" fontsize="10" font="Zurich Ex BT" /> 
+                        <words id="cal_param21" y="0" fontsize="10"/> 
+                        <words id="cal_param22" y="12" fontsize="10"/> 
+                        <words id="cal_param23" y="24" fontsize="10"/> 
+                        <words id="cal_param24" y="36" fontsize="10"/> 
+                        <words id="cal_param25" y="48" fontsize="10"/> 
                     </div>
                 </div>
             </div>
@@ -251,7 +246,7 @@ class Calibrator(AVGApp):
         parentNode.insertChild(self.mainNode, 0)
 
         self.coordCal = None
-        self.tracker = g_Player.getTracker()
+        self.tracker = player.getTracker()
         self.curParam = 0
         self.saveIndex = 0
         self.hideMainNodeTimeout = None
@@ -287,13 +282,13 @@ class Calibrator(AVGApp):
         self.tracker.setDebugImages(True, True)
         avg.fadeIn(self.mainNode, 400, 1)
         Bitmap = self.tracker.getImage(avg.IMG_DISTORTED)  # Why is this needed?
-        self.__onFrameID=g_Player.setOnFrameHandler(self.__onFrame)
+        self.__onFrameID = player.subscribe(player.ON_FRAME, self.__onFrame)
         #grandparent = self.parentNode.getParent()
         #if grandparent:
         #    grandparent.reorderChild(grandparent.indexOf(self.parentNode), grandparent.getNumChildren()-1)
         self.displayParams()
         if self.hideMainNodeTimeout:
-            g_Player.clearInterval(self.hideMainNodeTimeout)
+            player.clearInterval(self.hideMainNodeTimeout)
 
     def _leave(self):
         #unbind all calibrator keys - bind old keys
@@ -306,8 +301,8 @@ class Calibrator(AVGApp):
         #grandparent = self.parentNode.getParent()
         #if grandparent:
         #    grandparent.reorderChild(grandparent.indexOf(self.parentNode), 0)
-        self.hideMainNodeTimeout = g_Player.setTimeout(400, hideMainNode)
-        g_Player.clearInterval(self.__onFrameID)
+        self.hideMainNodeTimeout = player.setTimeout(400, hideMainNode)
+        player.clearInterval(self.__onFrameID)
 
     def reparent(self, newParent):
         """reparents the calibrator node; returns the old(!) parent node"""
@@ -321,7 +316,7 @@ class Calibrator(AVGApp):
         self.tracker.resetHistory()
         self.setNotification('')
         g_KbManager.pop()
-        g_Player.getElementByID('cal_params').opacity = 0.9
+        player.getElementByID('cal_params').opacity = 0.9
 
     def __clearNotification(self):
         self.__notificationTimer = None
@@ -329,12 +324,12 @@ class Calibrator(AVGApp):
 
     def __toggleGUI(self):
         self.__guiOpacity = 1 - self.__guiOpacity
-        g_Player.getElementByID('cal_gui').opacity = self.__guiOpacity
+        player.getElementByID('cal_gui').opacity = self.__guiOpacity
 
     def __onFrame(self):
         def showTrackerImage(trackerImageID, nodeID, size, pos=(0,0)):
             bitmap = self.tracker.getImage(trackerImageID)
-            node = g_Player.getElementByID(nodeID)
+            node = player.getElementByID(nodeID)
             node.setBitmap(bitmap)
             node.size = size
             if pos != (0,0):
@@ -354,8 +349,8 @@ class Calibrator(AVGApp):
         showTrackerImage(avg.IMG_CAMERA, "cal_camera", (160, 120))
         showTrackerImage(avg.IMG_NOHISTORY, "cal_nohistory", (160, 120))
         showTrackerImage(avg.IMG_HISTOGRAM, "cal_histogram", (160, 120))
-        fps = g_Player.getEffectiveFramerate()
-        g_Player.getElementByID("cal_fps").text = '%(val).2f' % {'val': fps} 
+        fps = player.getEffectiveFramerate()
+        player.getElementByID("cal_fps").text = '%(val).2f' % {'val': fps} 
         
     def __trackerSetDebugImages(self):
         self.appStarter.toggleTrackerImage()
@@ -394,7 +389,7 @@ class Calibrator(AVGApp):
     def __trackerSaveConfig(self):
         self.tracker.saveConfig()
         self.setNotification('Tracker configuration saved', 2000)
-        g_Log.trace(g_Log.APP, "Tracker configuration saved.")
+        avg.logger.info("Tracker configuration saved.")
               
     def __saveTrackerIMG(self):
         def saveTrackerImage(id, name):
@@ -408,7 +403,7 @@ class Calibrator(AVGApp):
         saveTrackerImage(avg.IMG_FINGERS, "fingers")
         saveTrackerImage(avg.IMG_HISTOGRAM, "histogram")
         self.setNotification('Tracker images dumped', 2000)
-        g_Log.trace(g_Log.APP, "Tracker images saved.")
+        avg.logger.info("Tracker images saved.")
     
     def __startCoordCalibration(self):
         assert(not self.coordCal)
@@ -432,24 +427,24 @@ class Calibrator(AVGApp):
         self.__onCalibrationSuccess = callback
 
     def deferredRefresh(self):
-        g_Player.setTimeout(1500, self.__deferredRefreshCB)
+        player.setTimeout(1500, self.__deferredRefreshCB)
         self.setNotification('Please wait for settlement')
         g_KbManager.push()
-        g_Player.getElementByID('cal_params').opacity = 0.3
+        player.getElementByID('cal_params').opacity = 0.3
 
     def setNotification(self, text, timeout=0):
-        g_Player.getElementByID('cal_notification').text = text
+        player.getElementByID('cal_notification').text = text
         if timeout:
             if self.__notificationTimer is not None:
-                g_Player.clearInterval(self.__notificationTimer)
+                player.clearInterval(self.__notificationTimer)
             
-            self.__notificationTimer = g_Player.setTimeout(timeout,
+            self.__notificationTimer = player.setTimeout(timeout,
                     self.__clearNotification)
 
     def displayParams(self):
         i = 0
         for Param in self.paramList:
-            Node = g_Player.getElementByID("cal_param"+str(i))
+            Node = player.getElementByID("cal_param"+str(i))
             Path = Param['path']
             Val = float(self.tracker.getParam(Path))
             Node.text = (Param['Name']+": "
