@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from libavg import avg
+from libavg import avg, player
 
 offset = None
 
@@ -25,13 +25,12 @@ def onMouseUp(event):
         node.releaseEventCapture()
         offset = None;
 
-player = avg.Player.get()
-
-player.loadFile("text.avg")
-node = player.getElementByID("HelloText")
-node.setEventHandler(avg.CURSORDOWN, avg.MOUSE, onMouseDown)
-node.setEventHandler(avg.CURSORMOTION, avg.MOUSE, onMouseMove)
-node.setEventHandler(avg.CURSORUP, avg.MOUSE, onMouseUp)
+canvas = player.createMainCanvas(size=(640,480))
+rootNode = canvas.getRootNode()
+node = avg.WordsNode(pos=(10,10), text="Hello World", parent=rootNode)
+node.subscribe(node.CURSOR_DOWN, onMouseDown)
+node.subscribe(node.CURSOR_MOTION, onMouseMove)
+node.subscribe(node.CURSOR_UP, onMouseUp)
 
 player.play()
 
