@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,19 +22,17 @@
 #
 
 import sys
-from libavg import avg
-import time
-
-Player = avg.Player.get()
+from libavg import player
 
 if len(sys.argv) ==1:
+    print
+    print "avg_showfile.py displays the contents of an avg file."
     print
     print "Usage: avg_showfile.py <avgfile>"
     print
     sys.exit(1)
 
-Player.loadFile(sys.argv[1])
-Player.setVBlankFramerate(1)
-Player.play()
+player.loadFile(sys.argv[1])
+player.play()
 
 
