@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 #include "../player/TouchEvent.h"
 #include "../player/Contact.h"
 #include "../player/TrackerInputDevice.h"
+#include "../player/Publisher.h"
 
 #include <boost/shared_ptr.hpp>
 #include <string>
@@ -82,29 +83,6 @@ void export_event()
     from_python_sequence<ContourSeq, variable_capacity_policy>();
     from_python_sequence<vector<EventPtr>, variable_capacity_policy>();
 
-    enum_<Event::Type>("Type")
-        .value("KEYUP", Event::KEYUP)
-        .value("KEYDOWN", Event::KEYDOWN)
-        .value("CURSORMOTION", Event::CURSORMOTION)
-        .value("CURSORUP", Event::CURSORUP)
-        .value("CURSORDOWN", Event::CURSORDOWN)
-        .value("CURSOROVER", Event::CURSOROVER)
-        .value("CURSOROUT", Event::CURSOROUT)
-        .value("CUSTOMEVENT", Event::CUSTOMEVENT)
-        .value("RESIZE", Event::RESIZE)
-        .value("QUIT", Event::QUIT)
-        .export_values()
-    ;
-
-    enum_<CursorEvent::Source>("Source")
-        .value("MOUSE", CursorEvent::MOUSE)
-        .value("TOUCH", CursorEvent::TOUCH)
-        .value("TRACK", CursorEvent::TRACK)
-        .value("CUSTOM", Event::CUSTOM)
-        .value("NONE", Event::NONE)
-        .export_values()
-    ;
-
     enum_<int>("KeyModifier")
         .value("KEYMOD_NONE", key::KEYMOD_NONE)
         .value("KEYMOD_LSHIFT", key::KEYMOD_LSHIFT)
@@ -126,17 +104,41 @@ void export_event()
         .export_values()
     ;
 
-    class_<Event, boost::noncopyable>("Event", init<Event::Type, Event::Source,
-            optional<int> >())
+    scope mainScope;
+
+    scope eventScope = class_<Event, boost::noncopyable>("Event", init<Event::Type,
+            Event::Source, optional<int> >())
         .add_property("type", &Event::getType)
         .add_property("source", &Event::getSource)
         .add_property("when", &Event::getWhen)
         .add_property("inputdevice", &Event::getInputDevice)
-        .add_property("inputdevicename",
-                      make_function(&Event::getInputDeviceName,
-                                    return_value_policy<copy_const_reference>()))
+        .add_property("inputdevicename", make_function(&Event::getInputDeviceName,
+                return_value_policy<copy_const_reference>()))
     ;
 
+    enum_<Event::Type>("Type")
+        .value("KEY_UP", Event::KEY_UP)
+        .value("KEY_DOWN", Event::KEY_DOWN)
+        .value("CURSOR_MOTION", Event::CURSOR_MOTION)
+        .value("CURSOR_UP", Event::CURSOR_UP)
+        .value("CURSOR_DOWN", Event::CURSOR_DOWN)
+        .value("CURSOR_OVER", Event::CURSOR_OVER)
+        .value("CURSOR_OUT", Event::CURSOR_OUT)
+        .value("CUSTOM_EVENT", Event::CUSTOM_EVENT)
+        .export_values()
+    ;
+
+    enum_<CursorEvent::Source>("Source")
+        .value("MOUSE", CursorEvent::MOUSE)
+        .value("TOUCH", CursorEvent::TOUCH)
+        .value("TRACK", CursorEvent::TRACK)
+        .value("CUSTOM", Event::CUSTOM)
+        .value("NONE", Event::NONE)
+        .export_values()
+    ;
+
+    scope oldScope1(mainScope);
+
     class_<CursorEvent, boost::shared_ptr<CursorEvent>, bases<Event> >("CursorEvent", 
             no_init)
         .add_property("source", &CursorEvent::getSource)
@@ -161,7 +163,7 @@ void export_event()
     
     class_<MouseEvent, bases<CursorEvent> >("MouseEvent",
             init<Event::Type, bool, bool, bool, const IntPoint&, int,
-                 optional<const DPoint&, int> >())
+                 optional<const glm::vec2&, int> >())
         .add_property("leftbuttonstate", &MouseEvent::getLeftButtonState)
         .add_property("middlebuttonstate", &MouseEvent::getMiddleButtonState)
         .add_property("rightbuttonstate", &MouseEvent::getRightButtonState)
@@ -169,7 +171,7 @@ void export_event()
     ;
 
     class_<TouchEvent, bases<CursorEvent> >("TouchEvent", init<int, Event::Type,
-            const IntPoint&, Event::Source, optional<const DPoint&> >())
+            const IntPoint&, Event::Source, optional<const glm::vec2&> >())
         .add_property("area", &TouchEvent::getArea)
         .add_property("orientation", &TouchEvent::getOrientation)
         .add_property("eccentricity", &TouchEvent::getEccentricity)
@@ -184,7 +186,8 @@ void export_event()
         .def("getContour", &TouchEvent::getContour)
         ;
 
-    class_<Contact, boost::shared_ptr<Contact> >("Contact", no_init)
+    object contactClass = class_<Contact, boost::shared_ptr<Contact>, bases<Publisher> >
+            ("Contact", no_init)
         .add_property("id", &Contact::getID)
         .add_property("age", &Contact::getAge)
         .add_property("distancefromstart", &Contact::getDistanceFromStart)
@@ -194,10 +197,8 @@ void export_event()
         .add_property("events", &Contact::getEvents)
         .def("connectListener", &Contact::connectListener)
         .def("disconnectListener", &Contact::disconnectListener)
-        .def("__hash__", &Contact::getHash)
-        .def(self == self)
-        .def(self != self)
         ;
+    exportMessages(contactClass, "Contact");
 
     enum_<TrackerImageID>("TrackerImageID")
         .value("IMG_CAMERA", TRACKER_IMG_CAMERA)
